@@ -1,6 +1,6 @@
 /* commands.c
  * Code for directives
- * Copyright (C) 1991-2002 Olly Betts
+ * Copyright (C) 1991-2003,2004 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include <assert.h>
 #include <limits.h>
+#include <stddef.h> /* for offsetof */
 
 #include "cavern.h"
 #include "commands.h"
@@ -39,17 +40,20 @@
 static void
 default_grade(settings *s)
 {
-   s->Var[Q_POS] = (real)sqrd(0.10);
-   s->Var[Q_LENGTH] = (real)sqrd(0.10);
-   s->Var[Q_COUNT] = (real)sqrd(0.10);
-   s->Var[Q_DX] = s->Var[Q_DY] = s->Var[Q_DZ] = (real)sqrd(0.10);
-   s->Var[Q_BEARING] = (real)sqrd(rad(1.0));
-   s->Var[Q_GRADIENT] = (real)sqrd(rad(1.0));
+   /* Values correspond to those in bcra5.svx */
+   s->Var[Q_POS] = (real)sqrd(0.05);
+   s->Var[Q_LENGTH] = (real)sqrd(0.05);
+   s->Var[Q_COUNT] = (real)sqrd(0.05);
+   s->Var[Q_DX] = s->Var[Q_DY] = s->Var[Q_DZ] = (real)sqrd(0.05);
+   s->Var[Q_BEARING] = (real)sqrd(rad(0.5));
+   s->Var[Q_GRADIENT] = (real)sqrd(rad(0.5));
+   s->Var[Q_BACKBEARING] = (real)sqrd(rad(0.5));
+   s->Var[Q_BACKGRADIENT] = (real)sqrd(rad(0.5));
    /* SD of plumbed legs (0.25 degrees?) */
    s->Var[Q_PLUMB] = (real)sqrd(rad(0.25));
    /* SD of level legs (0.25 degrees?) */
    s->Var[Q_LEVEL] = (real)sqrd(rad(0.25));
-   s->Var[Q_DEPTH] = (real)sqrd(0.10);
+   s->Var[Q_DEPTH] = (real)sqrd(0.05);
 }
 
 static void
@@ -64,7 +68,7 @@ default_case(settings *s)
    s->Case = LOWER;
 }
 
-reading default_order[] = { Fr, To, Tape, Comp, Clino, End };
+static reading default_order[] = { Fr, To, Tape, Comp, Clino, End };
 
 static void
 default_style(settings *s)
@@ -89,12 +93,12 @@ default_translate(settings *s)
       memcpy(t - 1, s->Translate - 1, sizeof(short) * 257);
       s->Translate = t;
    }
-/*  ASSERT(EOF==-1);*/ /* important, since we rely on this */
+/*  SVX_ASSERT(EOF==-1);*/ /* important, since we rely on this */
    t = s->Translate;
-   for (i = -1; i <= 255; i++) t[i] = 0;
-   for (i = '0'; i <= '9'; i++) t[i] |= SPECIAL_NAMES;
-   for (i = 'A'; i <= 'Z'; i++) t[i] |= SPECIAL_NAMES;
-   for (i = 'a'; i <= 'z'; i++) t[i] |= SPECIAL_NAMES;
+   memset(t - 1, 0, sizeof(short) * 257);
+   for (i = '0'; i <= '9'; i++) t[i] = SPECIAL_NAMES;
+   for (i = 'A'; i <= 'Z'; i++) t[i] = SPECIAL_NAMES;
+   for (i = 'a'; i <= 'z'; i++) t[i] = SPECIAL_NAMES;
 
    t['\t'] |= SPECIAL_BLANK;
    t[' '] |= SPECIAL_BLANK;
@@ -113,9 +117,13 @@ default_translate(settings *s)
    t['.'] |= SPECIAL_DECIMAL;
    t['-'] |= SPECIAL_MINUS;
    t['+'] |= SPECIAL_PLUS;
+#if 0 /* FIXME */
+   t['{'] |= SPECIAL_OPEN;
+   t['}'] |= SPECIAL_CLOSE;
+#endif
 }
 
-static void
+void
 default_units(settings *s)
 {
    int quantity;
@@ -125,9 +133,10 @@ default_units(settings *s)
       else
 	 s->units[quantity] = (real)1.0; /* metres */
    }
+   s->f_clino_percent = s->f_backclino_percent = fFalse;
 }
 
-static void
+void
 default_calib(settings *s)
 {
    int quantity;
@@ -147,8 +156,7 @@ extern void
 default_all(settings *s)
 {
    default_truncate(s);
-   s->f90Up = fFalse;
-   s->f0Eq = fFalse;
+   s->infer = 0;
    default_case(s);
    default_style(s);
    default_prefix(s);
@@ -159,7 +167,7 @@ default_all(settings *s)
    default_flags(s);
 }
 
-static char *buffer = NULL;
+char *buffer = NULL;
 static int buf_len;
 
 static char *ucbuffer = NULL;
@@ -168,22 +176,23 @@ static char *ucbuffer = NULL;
 extern void
 get_token(void)
 {
+   int i = -1;
+
    s_zero(&buffer);
    osfree(ucbuffer);
    skipblanks();
    while (isalpha(ch)) {
-      s_catchar(&buffer, &buf_len, ch);
+      s_catchar(&buffer, &buf_len, (char)ch);
       nextch();
    }
 
+   if (!buffer) s_catchar(&buffer, &buf_len, '\0');
+
    ucbuffer = osmalloc(buf_len);
-   {
-      int i = -1;
-      do {
-	 i++;
-	 ucbuffer[i] = toupper(buffer[i]);
-      } while (buffer[i]);
-   }
+   do {
+      i++;
+      ucbuffer[i] = toupper(buffer[i]);
+   } while (buffer[i]);
 #if 0
    printf("get_token() got `%s'\n", buffer);
 #endif
@@ -253,8 +262,18 @@ static sztok cmd_tab[] = {
      {NULL,        CMD_NULL}
 };
 
+/* masks for units which are length and angles respectively */
+#define LEN_UMASK (BIT(UNITS_METRES) | BIT(UNITS_FEET) | BIT(UNITS_YARDS))
+#define ANG_UMASK (BIT(UNITS_DEGS) | BIT(UNITS_GRADS) | BIT(UNITS_MINUTES))
+
+/* ordering must be the same as the units enum */
+static real factor_tab[] = {
+   1.0, METRES_PER_FOOT, (METRES_PER_FOOT*3.0),
+   (M_PI/180.0), (M_PI/200.0), 0.01, (M_PI/180.0/60.0)
+};
+
 static int
-get_units(void)
+get_units(unsigned long qmask, bool percent_ok)
 {
    static sztok utab[] = {
 	{"DEGREES",       UNITS_DEGS },
@@ -275,9 +294,17 @@ get_units(void)
    get_token();
    units = match_tok(utab, TABSIZE(utab));
    if (units == UNITS_NULL) {
-      compile_error(/*Unknown units `%s'*/35, buffer);
-   } else if (units == UNITS_PERCENT) {
-      NOT_YET;
+      compile_error_skip(/*Unknown units `%s'*/35, buffer);
+      return UNITS_NULL;
+   }
+   if (units == UNITS_PERCENT && percent_ok &&
+       !(qmask & ~(BIT(Q_GRADIENT)|BIT(Q_BACKGRADIENT)))) {
+      return units;
+   }
+   if (((qmask & LEN_QMASK) && !TSTBIT(LEN_UMASK, units)) ||
+       ((qmask & ANG_QMASK) && !TSTBIT(ANG_UMASK, units))) {
+      compile_error_skip(/*Invalid units `%s' for quantity*/37, buffer);
+      return UNITS_NULL;
    }
    return units;
 }
@@ -288,7 +315,10 @@ get_qlist(unsigned long mask_bad)
 {
    static sztok qtab[] = {
 	{"ALTITUDE",	 Q_DZ },
-	{"ANGLEOUTPUT",  Q_ANGLEOUTPUT },
+	{"BACKBEARING",  Q_BACKBEARING },
+	{"BACKCLINO",    Q_BACKGRADIENT },    /* alternative name */
+	{"BACKCOMPASS",  Q_BACKBEARING },     /* alternative name */
+	{"BACKGRADIENT", Q_BACKGRADIENT },
 	{"BEARING",      Q_BEARING },
 	{"CLINO",        Q_GRADIENT },    /* alternative name */
 	{"COMPASS",      Q_BEARING },     /* alternative name */
@@ -303,7 +333,6 @@ get_qlist(unsigned long mask_bad)
 	{"EASTING",      Q_DX },
 	{"GRADIENT",     Q_GRADIENT },
 	{"LENGTH",       Q_LENGTH },
-	{"LENGTHOUTPUT", Q_LENGTHOUTPUT },
 	{"LEVEL",        Q_LEVEL},
 	{"NORTHING",     Q_DY },
 	{"PLUMB",        Q_PLUMB},
@@ -327,15 +356,13 @@ get_qlist(unsigned long mask_bad)
       qmask |= BIT(tok);
       if (qmask != BIT(Q_DEFAULT) && (qmask & mask_bad)) {
 	 if (qmask & mask_bad & BIT(Q_DEFAULT)) strcpy(buffer, default_buf);
-	 compile_error(/*Unknown instrument `%s'*/39, buffer);
-	 skipline();
+	 compile_error_skip(/*Unknown instrument `%s'*/39, buffer);
 	 return 0;
       }
    }
 
    if (qmask == 0) {
-      compile_error(/*Unknown quantity `%s'*/34, buffer);
-      skipline();
+      compile_error_skip(/*Unknown quantity `%s'*/34, buffer);
    } else {
       set_pos(&fp);
    }
@@ -349,6 +376,7 @@ cmd_set(void)
 {
    static sztok chartab[] = {
 	{"BLANK",     SPECIAL_BLANK },
+/*FIXME	{"CLOSE",     SPECIAL_CLOSE }, */
 	{"COMMENT",   SPECIAL_COMMENT },
 	{"DECIMAL",   SPECIAL_DECIMAL },
 	{"EOL",       SPECIAL_EOL }, /* EOL won't work well */
@@ -356,6 +384,7 @@ cmd_set(void)
 	{"MINUS",     SPECIAL_MINUS },
 	{"NAMES",     SPECIAL_NAMES },
 	{"OMIT",      SPECIAL_OMIT },
+/*FIXME	{"OPEN",      SPECIAL_OPEN }, */
 	{"PLUS",      SPECIAL_PLUS },
 #ifndef NO_DEPRECATED
 	{"ROOT",      SPECIAL_ROOT },
@@ -370,8 +399,7 @@ cmd_set(void)
    mask = match_tok(chartab, TABSIZE(chartab));
 
    if (mask == SPECIAL_UNKNOWN) {
-      compile_error(/*Unknown character class `%s'*/42, buffer);
-      skipline();
+      compile_error_skip(/*Unknown character class `%s'*/42, buffer);
       return;
    }
 
@@ -380,7 +408,7 @@ cmd_set(void)
       if (root_depr_count < 5) {
 	 compile_warning(/*ROOT is deprecated*/25);
 	 if (++root_depr_count == 5)
-	    compile_warning(/*No further uses of this deprecated feature will be reported*/95);
+	    compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
       }
    }
 #endif
@@ -443,7 +471,7 @@ check_reentry(prefix *tag)
 	 if (reenter_depr_count < 5) {
 	    compile_warning(/*Reentering an existing prefix level is deprecated*/29);
 	    if (++reenter_depr_count == 5)
-	       compile_warning(/*No further uses of this deprecated feature will be reported*/95);
+	       compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
 	 }
 
 	 file.filename = tag->filename;
@@ -470,12 +498,12 @@ cmd_prefix(void)
    if (prefix_depr_count < 5) {
       compile_warning(/**prefix is deprecated - use *begin and *end instead*/6);
       if (++prefix_depr_count == 5)
-	 compile_warning(/*No further uses of this deprecated feature will be reported*/95);
+	 compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
    }
    tag = read_prefix_survey(fFalse, fTrue);
    pcs->Prefix = tag;
    check_reentry(tag);
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
    if (fUseNewFormat) {
       limb = get_twig(tag);
       if (limb->up->sourceval < 1) {
@@ -507,7 +535,7 @@ cmd_begin(void)
       check_reentry(tag);
       f_export_ok = fTrue;
 
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
       if (fUseNewFormat) {
 	 limb = get_twig(pcs->Prefix);
 	 if (limb->up->sourceval < 2) {
@@ -524,10 +552,12 @@ extern void
 free_settings(settings *p) {
    /* don't free default ordering or ordering used by parent */
    reading *order = p->ordering;
-   if (order != default_order && order != p->next->ordering) osfree(order);
+   if (order != default_order && (!p->next || order != p->next->ordering))
+      osfree(order);
 
    /* free Translate if not used by parent */
-   if (p->Translate != p->next->Translate) osfree(p->Translate - 1);
+   if (!p->next || p->Translate != p->next->Translate)
+      osfree(p->Translate - 1);
 
    osfree(p);
 }
@@ -539,24 +569,23 @@ cmd_end(void)
    prefix *tag, *tagBegin;
 
    pcsParent = pcs->next;
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
    if (fUseNewFormat) limb = get_twig(pcsParent->Prefix);
 #endif
 
    if (pcs->begin_lineno == 0) {
       if (pcsParent == NULL) {
 	 /* more ENDs than BEGINs */
-	 compile_error(/*No matching BEGIN*/192);
+	 compile_error_skip(/*No matching BEGIN*/192);
       } else {
-	 compile_error(/*END with no matching BEGIN in this file*/22);
+	 compile_error_skip(/*END with no matching BEGIN in this file*/22);
       }
-      skipline();
       return;
    }
 
    tagBegin = pcs->tag;
 
-   ASSERT(pcsParent);
+   SVX_ASSERT(pcsParent);
    free_settings(pcs);
    pcs = pcsParent;
 
@@ -566,10 +595,10 @@ cmd_end(void)
       if (tag) {
 	 if (!tagBegin) {
 	    /* "*begin" / "*end foo" */
-	    compile_error(/*Matching BEGIN tag has no prefix*/36);
+	    compile_error_skip(/*Matching BEGIN tag has no prefix*/36);
 	 } else {
 	    /* tag mismatch */
-	    compile_error(/*Prefix tag doesn't match BEGIN*/193);
+	    compile_error_skip(/*Prefix tag doesn't match BEGIN*/193);
 	 }
       } else {
 	 /* close tag omitted; open tag given */
@@ -592,6 +621,7 @@ cmd_fix(void)
    node *stn = NULL;
    static node *stnOmitAlready = NULL;
    real x, y, z;
+   int nx, ny, nz;
    bool fRef = 0;
    filepos fp;
 
@@ -606,17 +636,16 @@ cmd_fix(void)
       if (*ucbuffer) set_pos(&fp);
    }
 
-   x = read_numeric(fTrue);
+   x = read_numeric(fTrue, &nx);
    if (x == HUGE_REAL) {
       /* If the end of the line isn't blank, read a number after all to
        * get a more helpful error message */
-      if (!isEol(ch) && !isComm(ch)) x = read_numeric(fFalse);
+      if (!isEol(ch) && !isComm(ch)) x = read_numeric(fFalse, &nx);
    }
    if (x == HUGE_REAL) {
       if (stnOmitAlready) {
 	 if (fix_name != stnOmitAlready->name) {
-	    compile_error(/*More than one FIX command with no coordinates*/56);
-	    skipline();
+	    compile_error_skip(/*More than one FIX command with no coordinates*/56);
 	 } else {
 	    compile_warning(/*Same station fixed twice with no coordinates*/61);
 	 }
@@ -628,28 +657,28 @@ cmd_fix(void)
       stnOmitAlready = stn;
    } else {
       real sdx;
-      y = read_numeric(fFalse);
-      z = read_numeric(fFalse);
-      sdx = read_numeric(fTrue);
+      y = read_numeric(fFalse, &ny);
+      z = read_numeric(fFalse, &nz);
+      sdx = read_numeric(fTrue, NULL);
       if (sdx != HUGE_REAL) {
 	 real sdy, sdz;
 	 real cxy = 0, cyz = 0, czx = 0;
-	 sdy = read_numeric(fTrue);
+	 sdy = read_numeric(fTrue, NULL);
 	 if (sdy == HUGE_REAL) {
 	    /* only one variance given */
 	    sdy = sdz = sdx;
 	 } else {
-	    sdz = read_numeric(fTrue);
+	    sdz = read_numeric(fTrue, NULL);
 	    if (sdz == HUGE_REAL) {
 	       /* two variances given - horizontal & vertical */
 	       sdz = sdy;
 	       sdy = sdx;
 	    } else {
-	       cxy = read_numeric(fTrue);
+	       cxy = read_numeric(fTrue, NULL);
 	       if (cxy != HUGE_REAL) {
 		  /* covariances given */
-		  cyz = read_numeric(fFalse);
-		  czx = read_numeric(fFalse);
+		  cyz = read_numeric(fFalse, NULL);
+		  czx = read_numeric(fFalse, NULL);
 	       } else {
 		  cxy = 0;
 	       }
@@ -658,14 +687,31 @@ cmd_fix(void)
 	 stn = StnFromPfx(fix_name);
 	 if (!fixed(stn)) {
 	    node *fixpt = osnew(node);
-	    prefix *name = osnew(prefix);
+	    prefix *name;
+#ifdef CHASM3DX
+	    if (fUseNewFormat) {
+	       name = osnew(prefix);
+	       name->pos = osnew(pos);
+	    } else {
+	       /* only allocate the part of the structures we need... */
+	       name = /*(prefix *)*/osmalloc(offsetof(prefix, twig_link));
+	       name->pos = (pos *)osmalloc(offsetof(pos, id));
+	    }
+#else
+	    name = osnew(prefix);
+	    name->pos = osnew(pos);
+#endif
 	    name->ident = NULL;
 	    name->shape = 0;
 	    fixpt->name = name;
-	    name->pos = osnew(pos);
 	    name->stn = fixpt;
 	    name->up = NULL;
-	    name->min_export = name->max_export = 0;
+	    if (TSTBIT(pcs->infer, INFER_EXPORTS)) {
+	       name->min_export = USHRT_MAX;
+	    } else {
+	       name->min_export = 0;
+	    }
+	    name->max_export = 0;
 	    name->sflags = 0;
 	    add_stn_to_list(&stnlist, fixpt);
 	    POS(fixpt, 0) = x;
@@ -695,7 +741,7 @@ cmd_fix(void)
       POS(stn, 1) = y;
       POS(stn, 2) = z;
       fix(stn);
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
       if (fUseNewFormat) {
 	 fix_name->twig_link->from = fix_name; /* insert fixed point.. */
 	 fix_name->twig_link->to = NULL;
@@ -770,10 +816,9 @@ cmd_equate(void)
       name1 = read_prefix_stn_check_implicit(fTrue, fTrue);
       if (name1 == NULL) {
 	 if (fOnlyOneStn) {
-	    compile_error(/*Only one station in equate list*/33);
-	    skipline();
+	    compile_error_skip(/*Only one station in EQUATE command*/33);
 	 }
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
 	 if (fUseNewFormat) limb = get_twig(pcs->Prefix);
 #endif
 	 return;
@@ -794,7 +839,7 @@ report_missing_export(prefix *pfx, int depth)
    int i;
    for (i = depth + 1; i; i--) {
       survey = survey->up;
-      ASSERT(survey);
+      SVX_ASSERT(survey);
    }
    s = osstrdup(sprint_prefix(survey));
    if (survey->filename) {
@@ -826,10 +871,10 @@ cmd_export(void)
 	    p = p->up;
 	 }
 	 /* Something like: *export \foo, but we've excluded use of root */
-	 ASSERT(p);
+	 SVX_ASSERT(p);
       }
       /* *export \ or similar bogus stuff */
-      ASSERT(depth);
+      SVX_ASSERT(depth);
 #if 0
       printf("C min %d max %d depth %d pfx %s\n",
 	     pfx->min_export, pfx->max_export, depth, sprint_prefix(pfx));
@@ -838,7 +883,10 @@ cmd_export(void)
 	 /* not encountered *export for this name before */
 	 if (pfx->max_export > depth) report_missing_export(pfx, depth);
 	 pfx->min_export = pfx->max_export = depth;
-      } else {
+      } else if (pfx->min_export != USHRT_MAX) {
+	 /* FIXME: what to do if a station is marked for inferred exports
+	  * but is then explicitly exported?  Currently we just ignore the
+	  * explicit export... */
 	 if (pfx->min_export - 1 > depth) {
 	    report_missing_export(pfx, depth);
 	 } else if (pfx->min_export - 1 < depth) {
@@ -856,6 +904,10 @@ cmd_data(void)
 {
    static sztok dtab[] = {
 	{"ALTITUDE",	 Dz },
+	{"BACKBEARING",  BackComp },
+	{"BACKCLINO",    BackClino }, /* alternative name */
+	{"BACKCOMPASS",  BackComp }, /* alternative name */
+	{"BACKGRADIENT", BackClino },
 	{"BEARING",      Comp },
 	{"CLINO",        Clino }, /* alternative name */
 	{"COMPASS",      Comp }, /* alternative name */
@@ -887,11 +939,13 @@ cmd_data(void)
 #define MASK_stns BIT(Fr) | BIT(To) | BIT(Station)
 #define MASK_tape BIT(Tape) | BIT(FrCount) | BIT(ToCount) | BIT(Count)
 #define MASK_dpth BIT(FrDepth) | BIT(ToDepth) | BIT(Depth) | BIT(DepthChange)
+#define MASK_comp BIT(Comp) | BIT(BackComp)
+#define MASK_clin BIT(Clino) | BIT(BackClino)
 
-#define MASK_NORMAL MASK_stns | BIT(Dir) | MASK_tape | BIT(Comp) | BIT(Clino)
-#define MASK_DIVING MASK_stns | BIT(Dir) | MASK_tape | BIT(Comp) | MASK_dpth
+#define MASK_NORMAL MASK_stns | BIT(Dir) | MASK_tape | MASK_comp | MASK_clin
+#define MASK_DIVING MASK_stns | BIT(Dir) | MASK_tape | MASK_comp | MASK_dpth
 #define MASK_CARTESIAN MASK_stns | BIT(Dx) | BIT(Dy) | BIT(Dz)
-#define MASK_CYLPOLAR  MASK_stns | BIT(Dir) | MASK_tape | BIT(Comp) | MASK_dpth
+#define MASK_CYLPOLAR  MASK_stns | BIT(Dir) | MASK_tape | MASK_comp | MASK_dpth
 #define MASK_NOSURVEY MASK_stns
 
    /* readings which may be given for each style */
@@ -901,7 +955,7 @@ cmd_data(void)
 
    /* readings which may be omitted for each style */
    static const unsigned long mask_optional[] = {
-      BIT(Dir) | BIT(Clino),
+      BIT(Dir) | BIT(Clino) | BIT(BackClino),
       BIT(Dir),
       0,
       BIT(Dir),
@@ -953,8 +1007,7 @@ cmd_data(void)
    }
 
    if (style == STYLE_UNKNOWN) {
-      compile_error(/*Data style `%s' unknown*/65, buffer);
-      skipline();
+      compile_error_skip(/*Data style `%s' unknown*/65, buffer);
       return;
    }
 
@@ -968,7 +1021,7 @@ cmd_data(void)
 	 compile_warning(/*`*data %s %c ...' is deprecated - use `*data %s ...' instead*/104,
 			 buffer, ch, buffer);
 	 if (++data_depr_count == 5)
-	    compile_warning(/*No further uses of this deprecated feature will be reported*/95);
+	    compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
       }
       nextch();
    }
@@ -987,19 +1040,17 @@ cmd_data(void)
       }
       /* Note: an unknown token is reported as trailing garbage */
       if (!TSTBIT(mask_all[style], d)) {
-	 compile_error(/*Reading `%s' not allowed in data style `%s'*/63,
+	 compile_error_skip(/*Reading `%s' not allowed in data style `%s'*/63,
 		       buffer, style_name);
 	 osfree(style_name);
 	 osfree(new_order);
-	 skipline();
 	 return;
       }
       if (TSTBIT(mUsed, Newline) && TSTBIT(m_multi, d)) {
 	 /* e.g. "*data diving station newline tape depth compass" */
-	 compile_error(/*Reading `%s' must occur before NEWLINE*/225, buffer);
+	 compile_error_skip(/*Reading `%s' must occur before NEWLINE*/225, buffer);
 	 osfree(style_name);
 	 osfree(new_order);
-	 skipline();
 	 return;
       }
       /* Check for duplicates unless it's a special reading:
@@ -1007,12 +1058,13 @@ cmd_data(void)
        */
       if (!((BIT(Ignore) | BIT(End) | BIT(IgnoreAll)) & BIT(d))) {
 	 if (TSTBIT(mUsed, d)) {
-	    compile_error(/*Duplicate reading `%s'*/67, buffer);
+	    compile_error_skip(/*Duplicate reading `%s'*/67, buffer);
 	    osfree(style_name);
 	    osfree(new_order);
-	    skipline();
 	    return;
 	 } else {
+	    /* Check for previously listed readings which are incompatible
+	     * with this one - e.g. Count vs FrCount */
 	    bool fBad = fFalse;
 	    switch (d) {
 	     case Station:
@@ -1043,17 +1095,15 @@ cmd_data(void)
 	     case Newline:
 	       if (mUsed & ~m_multi) {
 		  /* e.g. "*data normal from to tape newline compass clino" */
-		  compile_error(/*NEWLINE can only be preceded by STATION, DEPTH, and COUNT*/226);
+		  compile_error_skip(/*NEWLINE can only be preceded by STATION, DEPTH, and COUNT*/226);
 		  osfree(style_name);
 		  osfree(new_order);
-		  skipline();
 		  return;
 	       }
 	       if (k == 0) {
-		  compile_error(/*NEWLINE can't be the first reading*/222);
+		  compile_error_skip(/*NEWLINE can't be the first reading*/222);
 		  osfree(style_name);
 		  osfree(new_order);
-		  skipline();
 		  return;
 	       }
 	       break;
@@ -1062,18 +1112,17 @@ cmd_data(void)
 	    }
 	    if (fBad) {
 	       /* Not entirely happy with phrasing this... */
-	       compile_error(/*Reading `%s' duplicates previous reading(s)*/219,
+	       compile_error_skip(/*Reading `%s' duplicates previous reading(s)*/77,
 			     buffer);
 	       osfree(style_name);
 	       osfree(new_order);
-	       skipline();
 	       return;
 	    }
 	    mUsed |= BIT(d); /* used to catch duplicates */
 	 }
       }
       if (k && new_order[k - 1] == IgnoreAll) {
-	 ASSERT(d == Newline);
+	 SVX_ASSERT(d == Newline);
 	 k--;
 	 d = IgnoreAllAndNewLine;
       }
@@ -1085,10 +1134,9 @@ cmd_data(void)
    } while (d != End);
 
    if (k >= 2 && new_order[k - 2] == Newline) {
-      compile_error(/*NEWLINE can't be the last reading*/223);
+      compile_error_skip(/*NEWLINE can't be the last reading*/223);
       osfree(style_name);
       osfree(new_order);
-      skipline();
       return;
    }
 
@@ -1106,7 +1154,7 @@ cmd_data(void)
        * *data normal station tape compass clino
        * (i.e. no newline, but interleaved readings)
        */
-      compile_error(/*Interleaved readings, but no NEWLINE*/224);
+      compile_error_skip(/*Interleaved readings, but no NEWLINE*/224);
       osfree(style_name);
       osfree(new_order);
       return;
@@ -1116,11 +1164,18 @@ cmd_data(void)
    printf("mUsed = 0x%x\n", mUsed);
 #endif
 
+   /* Check the supplied readings form a sufficient set. */
    if (mUsed & (BIT(Fr) | BIT(To)))
       mUsed |= BIT(Station);
    else if (TSTBIT(mUsed, Station))
       mUsed |= BIT(Fr) | BIT(To);
 
+   if (mUsed & (BIT(Comp) | BIT(BackComp)))
+      mUsed |= BIT(Comp) | BIT(BackComp);
+
+   if (mUsed & (BIT(Clino) | BIT(BackClino)))
+      mUsed |= BIT(Clino) | BIT(BackClino);
+
    if (mUsed & (BIT(FrDepth) | BIT(ToDepth)))
       mUsed |= BIT(Depth) | BIT(DepthChange);
    else if (TSTBIT(mUsed, Depth))
@@ -1142,9 +1197,9 @@ cmd_data(void)
 
    if (((mUsed &~ BIT(Newline)) | mask_optional[style]) != mask[style]) {
       /* Test should only fail with too few bits set, not too many */
-      ASSERT((((mUsed &~ BIT(Newline)) | mask_optional[style])
+      SVX_ASSERT((((mUsed &~ BIT(Newline)) | mask_optional[style])
 	      &~ mask[style]) == 0);
-      compile_error(/*Too few readings for data style `%s'*/64, style_name);
+      compile_error_skip(/*Too few readings for data style `%s'*/64, style_name);
       osfree(style_name);
       osfree(new_order);
       return;
@@ -1161,16 +1216,6 @@ cmd_data(void)
    osfree(style_name);
 }
 
-/* masks for units which are length and angles respectively */
-#define LEN_UMASK (BIT(UNITS_METRES) | BIT(UNITS_FEET) | BIT(UNITS_YARDS))
-#define ANG_UMASK (BIT(UNITS_DEGS) | BIT(UNITS_GRADS) | BIT(UNITS_MINUTES))
-
-/* ordering must be the same as the units enum */
-static real factor_tab[] = {
-   1.0, METRES_PER_FOOT, (METRES_PER_FOOT*3.0),
-   (M_PI/180.0), (M_PI/200.0), 0.0, (M_PI/180.0/60.0)
-};
-
 static void
 cmd_units(void)
 {
@@ -1186,21 +1231,22 @@ cmd_units(void)
       return;
    }
 
-   factor = read_numeric(fTrue);
-   if (factor == HUGE_REAL) factor = (real)1.0;
-
-   units = get_units();
-   if (units == UNITS_NULL) return;
-   factor *= factor_tab[units];
-
-   if (((qmask & LEN_QMASK) && !TSTBIT(LEN_UMASK, units)) ||
-       ((qmask & ANG_QMASK) && !TSTBIT(ANG_UMASK, units))) {
-      compile_error(/*Invalid units `%s' for quantity*/37, buffer);
+   factor = read_numeric(fTrue, NULL);
+   if (factor == 0.0) {
+      compile_error_skip(/**UNITS factor must be non-zero*/200);
       return;
    }
-   if (qmask & (BIT(Q_LENGTHOUTPUT) | BIT(Q_ANGLEOUTPUT)) ) {
-      NOT_YET;
-   }
+   if (factor == HUGE_REAL) factor = (real)1.0;
+
+   units = get_units(qmask, fTrue);
+   if (units == UNITS_NULL) return;
+   if (TSTBIT(qmask, Q_GRADIENT))
+      pcs->f_clino_percent = (units == UNITS_PERCENT);
+   if (TSTBIT(qmask, Q_BACKGRADIENT))
+      pcs->f_backclino_percent = (units == UNITS_PERCENT);
+
+   factor *= factor_tab[units];
+
    for (quantity = 0, m = BIT(quantity); m <= qmask; quantity++, m <<= 1)
       if (qmask & m) pcs->units[quantity] = factor;
 }
@@ -1212,8 +1258,7 @@ cmd_calibrate(void)
    unsigned long qmask, m;
    int quantity;
 
-   qmask = get_qlist(BIT(Q_DEFAULT)|BIT(Q_POS)|BIT(Q_LENGTHOUTPUT)|
-		BIT(Q_ANGLEOUTPUT)|BIT(Q_PLUMB)|BIT(Q_LEVEL));
+   qmask = get_qlist(BIT(Q_DEFAULT)|BIT(Q_POS)|BIT(Q_PLUMB)|BIT(Q_LEVEL));
    if (!qmask) return; /* error already reported */
 
    if (qmask == BIT(Q_DEFAULT)) {
@@ -1222,19 +1267,17 @@ cmd_calibrate(void)
    }
 
    if (((qmask & LEN_QMASK)) && ((qmask & ANG_QMASK))) {
-      compile_error(/*Can't calibrate angular and length quantities together*/227);
-      skipline();
+      compile_error_skip(/*Can't calibrate angular and length quantities together*/227);
       return;
    }
 
-   z = read_numeric(fFalse);
-   sc = read_numeric(fTrue);
+   z = read_numeric(fFalse, NULL);
+   sc = read_numeric(fTrue, NULL);
    if (sc == HUGE_REAL) sc = (real)1.0;
    /* check for declination scale */
    /* perhaps "*calibrate declination XXX" should be "*declination XXX" ? */
    if (TSTBIT(qmask, Q_DECLINATION) && sc != 1.0) {
-      compile_error(/*Scale factor must be 1.0 for DECLINATION*/40);
-      skipline();
+      compile_error_skip(/*Scale factor must be 1.0 for DECLINATION*/40);
       return;
    }
    for (quantity = 0, m = BIT(quantity); m <= qmask; quantity++, m <<= 1) {
@@ -1260,7 +1303,7 @@ cmd_default(void)
    if (default_depr_count < 5) {
       compile_warning(/**DEFAULT is deprecated - use *CALIBRATE/DATA/SD/UNITS with argument DEFAULT instead*/20);
       if (++default_depr_count == 5)
-	 compile_warning(/*No further uses of this deprecated feature will be reported*/95);
+	 compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
    }
 
    get_token();
@@ -1276,7 +1319,7 @@ cmd_default(void)
       default_units(pcs);
       break;
     default:
-      compile_error(/*Unknown setting `%s'*/41, buffer);
+      compile_error_skip(/*Unknown setting `%s'*/41, buffer);
    }
 }
 #endif
@@ -1309,7 +1352,7 @@ cmd_include(void)
 #ifndef NO_DEPRECATED
    root = root_store; /* and restore root */
 #endif
-#ifdef NEW3DFORMAT
+#ifdef CHASM3DX
    if (fUseNewFormat) limb = get_twig(root);
 #endif
    ch = ch_store;
@@ -1325,26 +1368,20 @@ cmd_sd(void)
    int units;
    unsigned long qmask, m;
    int quantity;
-   qmask = get_qlist(BIT(Q_ANGLEOUTPUT)|BIT(Q_LENGTHOUTPUT)|BIT(Q_DECLINATION));
+   qmask = get_qlist(BIT(Q_DECLINATION));
    if (!qmask) return; /* no quantities found - error already reported */
 
    if (qmask == BIT(Q_DEFAULT)) {
       default_grade(pcs);
       return;
    }
-   sd = read_numeric(fFalse);
+   sd = read_numeric(fFalse, NULL);
    if (sd <= (real)0.0) {
-      compile_error(/*Standard deviation must be positive*/216);
+      compile_error_skip(/*Standard deviation must be positive*/48);
       return;
    }
-   units = get_units();
+   units = get_units(qmask, fFalse);
    if (units == UNITS_NULL) return;
-   if (((qmask & LEN_QMASK) && !TSTBIT(LEN_UMASK, units)) ||
-       ((qmask & ANG_QMASK) && !TSTBIT(ANG_UMASK, units))) {
-      compile_error(/*Invalid units `%s' for quantity*/37, buffer);
-      skipline();
-      return;
-   }
 
    sd *= factor_tab[units];
    variance = sqrd(sd);
@@ -1386,53 +1423,50 @@ cmd_case(void)
    if (setting != -1) {
       pcs->Case = setting;
    } else {
-      compile_error(/*Found `%s', expecting `PRESERVE', `TOUPPER', or `TOLOWER'*/10,
+      compile_error_skip(/*Found `%s', expecting `PRESERVE', `TOUPPER', or `TOLOWER'*/10,
 		    buffer);
-      skipline();
    }
 }
 
 static sztok infer_tab[] = {
-     {"EQUATES",    2},
-     {"PLUMBS",     1},
-     {NULL,      0}
+     { "EQUATES",	INFER_EQUATES },
+     { "EXPORTS",	INFER_EXPORTS },
+     { "PLUMBS",	INFER_PLUMBS },
+#if 0 /* FIXME */
+     { "SUBSURVEYS",	INFER_SUBSURVEYS },
+#endif
+     { NULL,		INFER_NULL }
 };
 
 static sztok onoff_tab[] = {
-     {"OFF", 0},
-     {"ON",  1},
-     {NULL,       -1}
+     { "OFF", 0 },
+     { "ON",  1 },
+     { NULL, -1 }
 };
 
 static void
 cmd_infer(void)
 {
-   int setting;
+   infer_what setting;
    int on;
    get_token();
    setting = match_tok(infer_tab, TABSIZE(infer_tab));
-   if (setting == -1) {
-      compile_error(/*Found `%s', expecting `PLUMBS'*/31, buffer);
-      skipline();
+   if (setting == INFER_NULL) {
+      compile_error_skip(/*Found `%s', expecting `EQUATES', `EXPORTS', or `PLUMBS'*/31, buffer);
       return;
    }
    get_token();
    on = match_tok(onoff_tab, TABSIZE(onoff_tab));
    if (on == -1) {
-      compile_error(/*Found `%s', expecting `ON' or `OFF'*/32, buffer);
-      skipline();
+      compile_error_skip(/*Found `%s', expecting `ON' or `OFF'*/32, buffer);
       return;
    }
 
-   switch (setting) {
-    case 2:
-      pcs->f0Eq = on;
-      break;
-    case 1:
-      pcs->f90Up = on;
-      break;
-    default:
-      BUG("unexpected case");
+   if (on) {
+      pcs->infer |= BIT(setting);
+      if (setting == INFER_EXPORTS) fExportUsed = fTrue;
+   } else {
+      pcs->infer &= ~BIT(setting);
    }
 }
 
@@ -1544,15 +1578,14 @@ handle_command(void)
    cmdtok = match_tok(cmd_tab, TABSIZE(cmd_tab));
 
    if (cmdtok < 0 || cmdtok >= (int)(sizeof(cmd_funcs) / sizeof(cmd_fn))) {
-      compile_error(/*Unknown command `%s'*/12, buffer);
-      skipline();
+      compile_error_skip(/*Unknown command `%s'*/12, buffer);
       return;
    }
 
    switch (cmdtok) {
     case CMD_EXPORT:
       if (!f_export_ok)
-	 compile_error(/**EXPORT must immediately follow *BEGIN*/57);
+	 compile_error(/**EXPORT must immediately follow `*BEGIN <SURVEY>'*/57);
       break;
     case CMD_COPYRIGHT:
     case CMD_DATE:
