@@ -271,8 +271,7 @@ public:
 
     void ForceRefresh();
 
-    void RefreshLine(const Point &a1, const Point &b1,
-		     const Point &a2, const Point &b2);
+    void RefreshLine(const Point &a, const Point &b, const Point &c);
  
     void SetHere();
     void SetHere(Double x, Double y, Double z);
@@ -386,6 +385,7 @@ public:
 
     void OnPrint(const wxString &filename, const wxString &title,
 		 const wxString &datestamp);
+    bool OnExport(const wxString &filename);
 
 private:
     DECLARE_EVENT_TABLE()
