@@ -34,29 +34,16 @@
 #include "img.h"
 #include "namecmp.h"
 #include "printwx.h"
+#include "filename.h"
 
 #include <wx/confbase.h>
+#include <wx/regex.h>
 
 #include <float.h>
 #include <functional>
 #include <stack>
 
-#ifdef HAVE_REGEX_H
-# include <regex.h>
-#else
-extern "C" {
-# ifdef HAVE_RXPOSIX_H
-#  include <rxposix.h>
-# elif defined(HAVE_RX_RXPOSIX_H)
-#  include <rx/rxposix.h>
-# elif !defined(REG_NOSUB)
-// we must be using wxWindows built-in regexp functions
-#  define REG_NOSUB wxRE_NOSUB
-#  define REG_ICASE wxRE_ICASE
-#  define REG_EXTENDED wxRE_EXTENDED
-# endif
-}
-#endif
+using namespace std;
 
 const int NUM_DEPTH_COLOURS = 13; // up to 13
 
@@ -114,6 +101,7 @@ BEGIN_EVENT_TABLE(MainFrm, wxFrame)
     EVT_MENU(menu_FILE_OPEN, MainFrm::OnOpen)
     EVT_MENU(menu_FILE_PRINT, MainFrm::OnPrint)
     EVT_MENU(menu_FILE_PAGE_SETUP, MainFrm::OnPageSetup)
+    EVT_MENU(menu_FILE_EXPORT, MainFrm::OnExport)
     EVT_MENU(menu_FILE_QUIT, MainFrm::OnQuit)
     EVT_MENU_RANGE(wxID_FILE1, wxID_FILE9, MainFrm::OnMRUFile)
 
@@ -166,6 +154,7 @@ BEGIN_EVENT_TABLE(MainFrm, wxFrame)
     EVT_MENU(menu_HELP_ABOUT, MainFrm::OnAbout)
 
     EVT_UPDATE_UI(menu_FILE_PRINT, MainFrm::OnPrintUpdate)
+    EVT_UPDATE_UI(menu_FILE_EXPORT, MainFrm::OnExportUpdate)
     EVT_UPDATE_UI(menu_ROTATION_START, MainFrm::OnStartRotationUpdate)
     EVT_UPDATE_UI(menu_ROTATION_TOGGLE, MainFrm::OnToggleRotationUpdate)
     EVT_UPDATE_UI(menu_ROTATION_STOP, MainFrm::OnStopRotationUpdate)
@@ -285,7 +274,7 @@ MainFrm::MainFrm(const wxString& title, const wxPoint& pos, const wxSize& size)
     CreateToolBar();
     CreateSidePanel();
 
-#ifdef __X__
+#ifdef __X__ // wxMotif or wxX11
     int x;
     int y;
     GetSize(&x, &y);
@@ -327,6 +316,8 @@ void MainFrm::CreateMenuBar()
     filemenu->Append(menu_FILE_PRINT, GetTabMsg(/*@Print...##Ctrl+P*/380));
     filemenu->Append(menu_FILE_PAGE_SETUP, GetTabMsg(/*P@age Setup...*/381));
     filemenu->AppendSeparator();
+    filemenu->Append(menu_FILE_EXPORT, "Export as..."); // FIXME TRANSLATE
+    filemenu->AppendSeparator();
     filemenu->Append(menu_FILE_QUIT, GetTabMsg(/*@Quit##Ctrl+Q*/221));
 
     m_history.UseMenu(filemenu);
@@ -1069,17 +1060,9 @@ void MainFrm::OpenFile(const wxString& file, wxString survey, bool delay)
 	m_history.Save(*b);
 	b->Flush();
 
-	if (delay) {
-	    m_Gfx->InitialiseOnNextResize();
-	}
-	else {
-	    m_Gfx->Initialise();
-	}
-
-	m_Panel->Show(true);
 	int x;
 	int y;
-	GetSize(&x, &y);
+	GetClientSize(&x, &y);
 	if (x < 600)
 	    x /= 3;
 	else if (x < 1000)
@@ -1088,8 +1071,15 @@ void MainFrm::OpenFile(const wxString& file, wxString survey, bool delay)
 	    x /= 5;
 
 	m_Splitter->SplitVertically(m_Panel, m_Gfx, x);
+	m_SashPosition = x; // Save width of panel.
+
+	if (delay) {
+	    m_Gfx->InitialiseOnNextResize();
+	} else {
+	    m_Gfx->Initialise();
+	}
+	m_Panel->Show(true);
 
-	m_SashPosition = m_Splitter->GetSashPosition(); // save width of panel
 	m_Gfx->SetFocus();
     }
 }
@@ -1160,6 +1150,22 @@ void MainFrm::OnPageSetup(wxCommandEvent&)
     m_pageSetupData = pageSetupDialog.GetPageSetupData();
 }
 
+void MainFrm::OnExport(wxCommandEvent&)
+{
+    char *baseleaf = baseleaf_from_fnm(m_File.c_str());
+    wxFileDialog dlg(this, wxString("Export as:"), "",
+		     wxString(baseleaf),
+		     "DXF files|*.dxf|SVG files|*.svg|Sketch files|*.sk|Compass PLT for use with Carto|*.plt",
+		     wxSAVE|wxOVERWRITE_PROMPT);
+    free(baseleaf);
+    if (dlg.ShowModal() == wxID_OK) {
+	wxString fnm = dlg.GetPath();
+	if (!m_Gfx->OnExport(fnm)) {
+	    wxGetApp().ReportError(wxString::Format("Couldn't write file `%s'", fnm.c_str()));
+	}
+    }
+}
+
 void MainFrm::OnQuit(wxCommandEvent&)
 {
     exit(0);
@@ -1313,7 +1319,9 @@ void MainFrm::DisplayTreeInfo(const wxTreeItemData* item)
     const TreeData* data = static_cast<const TreeData*>(item);
     if (data) {
 	if (data->IsStation()) {
-	    ShowInfo(data->GetLabel());
+	    const LabelInfo * l = data->GetLabel();
+	    ShowInfo(l);
+	    m_Gfx->SetHere(l->x, l->y, l->z);
 	} else {
 	    m_StnName->SetLabel("");
 	    m_StnCoords->SetLabel("");
@@ -1322,6 +1330,7 @@ void MainFrm::DisplayTreeInfo(const wxTreeItemData* item)
 	    m_Dist1->SetLabel("");
 	    m_Dist2->SetLabel("");
 	    m_Dist3->SetLabel("");
+	    m_Gfx->SetHere();
 	}
     }
 }
@@ -1333,6 +1342,9 @@ void MainFrm::TreeItemSelected(wxTreeItemData* item)
     if (data && data->IsStation()) {
 	const LabelInfo* label = data->GetLabel();
 	m_Gfx->CentreOn(label->x, label->y, label->z);
+	m_Gfx->SetThere(label->x, label->y, label->z);
+    } else {
+	m_Gfx->SetThere();
     }
 
     m_Dist1->SetLabel("");
@@ -1345,19 +1357,19 @@ void MainFrm::OnFind(wxCommandEvent&)
     wxBusyCursor hourglass;
     // Find stations specified by a string or regular expression.
 
-    wxString str = m_FindBox->GetValue();
-    int cflags = REG_NOSUB;
+    wxString pattern = m_FindBox->GetValue();
+    int re_flags = wxRE_NOSUB;
 
     if (true /* case insensitive */) {
-	cflags |= REG_ICASE;
+	re_flags |= wxRE_ICASE;
     }
 
     if (m_RegexpCheckBox->GetValue()) {
-	cflags |= REG_EXTENDED;
+	re_flags |= wxRE_EXTENDED;
     } else if (false /* simple glob-style */) {
 	wxString pat;
-	for (size_t i = 0; i < str.size(); i++) {
-	   char ch = str[i];
+	for (size_t i = 0; i < pattern.size(); i++) {
+	   char ch = pattern[i];
 	   // ^ only special at start; $ at end.  But this is simpler...
 	   switch (ch) {
 	    case '^': case '$': case '.': case '[': case '\\':
@@ -1374,11 +1386,12 @@ void MainFrm::OnFind(wxCommandEvent&)
 	      pat += ch;
 	   }
 	}
-	str = pat;
+	pattern = pat;
+	re_flags |= wxRE_BASIC;
     } else {
 	wxString pat;
-	for (size_t i = 0; i < str.size(); i++) {
-	   char ch = str[i];
+	for (size_t i = 0; i < pattern.size(); i++) {
+	   char ch = pattern[i];
 	   // ^ only special at start; $ at end.  But this is simpler...
 	   switch (ch) {
 	    case '^': case '$': case '*': case '.': case '[': case '\\':
@@ -1386,23 +1399,21 @@ void MainFrm::OnFind(wxCommandEvent&)
 	   }
 	   pat += ch;
 	}
-	str = pat;
+	pattern = pat;
+	re_flags |= wxRE_BASIC;
     }
 
     if (!true /* !substring */) {
 	// FIXME "0u" required to avoid compilation error with g++-3.0
-	if (str.empty() || str[0u] != '^') str = '^' + str;
-	if (str[str.size() - 1] != '$') str = str + '$';
+	if (pattern.empty() || pattern[0u] != '^') pattern = '^' + pattern;
+	if (pattern[pattern.size() - 1] != '$') pattern = pattern + '$';
     }
 
-    regex_t buffer;
-    int errcode = regcomp(&buffer, str.c_str(), cflags);
-    if (errcode) {
-	size_t len = regerror(errcode, &buffer, NULL, 0);
-	char *msg = new char[len];
-	regerror(errcode, &buffer, msg, len);
-	wxGetApp().ReportError(msg);
-	delete[] msg;
+    wxRegEx regex;
+    if (!regex.Compile(pattern, re_flags)) {
+	wxString m;
+	m.Printf(msg(/*Invalid regular expression: %s*/404), pattern.c_str());
+	wxGetApp().ReportError(m);
 	return;
     }
 
@@ -1412,7 +1423,7 @@ void MainFrm::OnFind(wxCommandEvent&)
     while (pos != m_Labels.end()) {
 	LabelInfo* label = *pos++;
 
-	if (regexec(&buffer, label->text.c_str(), 0, NULL, 0) == 0) {
+	if (regex.Matches(label->text)) {
 	    label->flags |= LFLAG_HIGHLIGHTED;
 	    found++;
 	} else {
@@ -1420,8 +1431,6 @@ void MainFrm::OnFind(wxCommandEvent&)
 	}
     }
 
-    regfree(&buffer);
-
     m_Found->SetLabel(wxString::Format(msg(/*%d found*/331), found));
 #ifdef _WIN32
     m_Found->Refresh(); // FIXME
