@@ -1,6 +1,6 @@
 /* message.c
  * Fairly general purpose message and error routines
- * Copyright (C) 1993-2002 Olly Betts
+ * Copyright (C) 1993-2003,2004 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <errno.h>
 #include <locale.h>
 
+#include "cmdline.h"
 #include "whichos.h"
 #include "filename.h"
 #include "message.h"
@@ -53,9 +54,10 @@ static jmp_buf jmpbufSignal;
 #endif
 
 #if (OS==WIN32)
+# define WIN32_LEAN_AND_MEAN
 # include <windows.h>
 #elif (OS==MSDOS)
-#include <dos.h>
+# include <dos.h>
 # ifdef __DJGPP__
 #  include <dpmi.h>
 #  include <go32.h>
@@ -64,6 +66,9 @@ static jmp_buf jmpbufSignal;
 #elif (OS==RISCOS)
 # include "oslib/wimpreadsy.h"
 # include "oslib/territory.h"
+#elif (OS==UNIX)
+# include <sys/types.h>
+# include <sys/stat.h>
 #endif
 
 /* For funcs which want to be immune from messing around with different
@@ -94,10 +99,10 @@ static const char tombstone[TOMBSTONE_SIZE] = "012345\xfftombstone";
 /* malloc with error catching if it fails. Also allows us to write special
  * versions easily eg for DOS EMS or MS Windows.
  */
-void FAR *
+void Far *
 osmalloc(OSSIZE_T size)
 {
-   void FAR *p;
+   void Far *p;
 #ifdef TOMBSTONES
    size += TOMBSTONE_SIZE * 2;
    p = malloc(size);
@@ -116,7 +121,7 @@ osmalloc(OSSIZE_T size)
 }
 
 /* realloc with error catching if it fails. */
-void FAR *
+void Far *
 osrealloc(void *p, OSSIZE_T size)
 {
    /* some pre-ANSI realloc implementations don't cope with a NULL pointer */
@@ -154,7 +159,7 @@ osrealloc(void *p, OSSIZE_T size)
    return p;
 }
 
-char FAR *
+char Far *
 osstrdup(const char *str)
 {
    char *p;
@@ -199,7 +204,7 @@ static int sigReceived;
 # define RETSIGTYPE void
 #endif
 
-static CDECL RETSIGTYPE FAR
+static CDECL RETSIGTYPE Far
 report_sig(int sig)
 {
    sigReceived = sig;
@@ -211,12 +216,10 @@ init_signals(void)
 {
    int en;
    if (!setjmp(jmpbufSignal)) {
-#if 0 /* disable these to get a core dump */
       signal(SIGABRT, report_sig); /* abnormal termination eg abort() */
       signal(SIGFPE,  report_sig); /* arithmetic error eg /0 or overflow */
       signal(SIGILL,  report_sig); /* illegal function image eg illegal instruction */
       signal(SIGSEGV, report_sig); /* illegal storage access eg access outside memory limits */
-#endif
 # ifdef SIGSTAK /* only on RISC OS AFAIK */
       signal(SIGSTAK, report_sig); /* stack overflow */
 # endif
@@ -289,6 +292,7 @@ default_charset(void)
 #endif
       if (p == 437) return CHARSET_DOSCP437;
       if (p == 850) return CHARSET_DOSCP850;
+      if (p == 912) return CHARSET_ISO_8859_2;
    }
    return CHARSET_USASCII;
 #elif (OS==WIN32)
@@ -332,7 +336,7 @@ default_charset(void)
 	 size_t cnt;
 
 	 for (cnt = 0; cnt < name_len; ++cnt)
-	    if (isalpha(chset[cnt])) {
+	    if (isalpha((unsigned char)chset[cnt])) {
 	       only_digit = 0;
 	       break;
 	    }
@@ -346,9 +350,11 @@ default_charset(void)
 	       iso:
 	       if (strncmp(chset, "8859", 4) == 0) {
 		  chset += 4;
-		  while (chset < p && *chset && !isdigit(*chset)) chset++;
+		  while (chset < p && *chset && !isdigit((unsigned char)*chset))
+		     chset++;
 		  switch (atoi(chset)) {
 		   case 1: return CHARSET_ISO_8859_1;
+		   case 2: return CHARSET_ISO_8859_2;
 		   case 15: return CHARSET_ISO_8859_15;
 		   default: return CHARSET_USASCII;
 		  }
@@ -358,7 +364,8 @@ default_charset(void)
 	  case 'u':
 	    if (tolower(chset[1]) == 't' && tolower(chset[2]) == 'f') {
 	       chset += 3;
-	       while (chset < p && *chset && !isdigit(*chset)) chset++;
+	       while (chset < p && *chset && !isdigit((unsigned char)*chset))
+		  chset++;
 	       switch (atoi(chset)) {
 		case 8: return CHARSET_UTF8;
 		default: return CHARSET_USASCII;
@@ -401,6 +408,81 @@ add_unicode(int charset, unsigned char *p, int value)
 	 return 1;
       }
       break;
+   case CHARSET_ISO_8859_2:
+      if (value >= 0xa0) {
+	 int v = 0;
+	 switch (value) {
+	    case 0xa0: case 0xa4: case 0xa7: case 0xa8: case 0xad: case 0xb0:
+	    case 0xb4: case 0xb8: case 0xc1: case 0xc2: case 0xc4: case 0xc7:
+	    case 0xc9: case 0xcb: case 0xcd: case 0xce: case 0xd3: case 0xd4:
+	    case 0xd6: case 0xd7: case 0xda: case 0xdc: case 0xdd: case 0xdf:
+	    case 0xe1: case 0xe2: case 0xe4: case 0xe7: case 0xe9: case 0xeb:
+	    case 0xed: case 0xee: case 0xf3: case 0xf4: case 0xf6: case 0xf7:
+	    case 0xfa: case 0xfc: case 0xfd:
+	       v = value; break;
+	    case 0x104: v = '\xa1'; break;
+	    case 0x2d8: v = '\xa2'; break;
+	    case 0x141: v = '\xa3'; break;
+	    case 0x13d: v = '\xa5'; break;
+	    case 0x15a: v = '\xa6'; break;
+	    case 0x160: v = '\xa9'; break;
+	    case 0x15e: v = '\xaa'; break; /* Scedil */
+	    case 0x164: v = '\xab'; break;
+	    case 0x179: v = '\xac'; break;
+	    case 0x17d: v = '\xae'; break;
+	    case 0x17b: v = '\xaf'; break;
+	    case 0x105: v = '\xb1'; break;
+	    case 0x2db: v = '\xb2'; break;
+	    case 0x142: v = '\xb3'; break;
+	    case 0x13e: v = '\xb5'; break;
+	    case 0x15b: v = '\xb6'; break;
+	    case 0x2c7: v = '\xb7'; break;
+	    case 0x161: v = '\xb9'; break;
+	    case 0x15f: v = '\xba'; break; /* scedil */
+	    case 0x165: v = '\xbb'; break;
+	    case 0x17a: v = '\xbc'; break;
+	    case 0x2dd: v = '\xbd'; break;
+	    case 0x17e: v = '\xbe'; break;
+	    case 0x17c: v = '\xbf'; break;
+	    case 0x154: v = '\xc0'; break;
+	    case 0x102: v = '\xc3'; break;
+	    case 0x139: v = '\xc5'; break;
+	    case 0x106: v = '\xc6'; break;
+	    case 0x10c: v = '\xc8'; break;
+	    case 0x118: v = '\xca'; break;
+	    case 0x11a: v = '\xcc'; break;
+	    case 0x10e: v = '\xcf'; break;
+	    case 0x110: v = '\xd0'; break;
+	    case 0x143: v = '\xd1'; break;
+	    case 0x147: v = '\xd2'; break;
+	    case 0x150: v = '\xd5'; break;
+	    case 0x158: v = '\xd8'; break;
+	    case 0x16e: v = '\xd9'; break;
+	    case 0x170: v = '\xdb'; break;
+	    case 0x162: v = '\xde'; break; /* &Tcedil; */
+	    case 0x155: v = '\xe0'; break;
+	    case 0x103: v = '\xe3'; break;
+	    case 0x13a: v = '\xe5'; break;
+	    case 0x107: v = '\xe6'; break;
+	    case 0x10d: v = '\xe8'; break;
+	    case 0x119: v = '\xea'; break;
+	    case 0x11b: v = '\xec'; break;
+	    case 0x10f: v = '\xef'; break;
+	    case 0x111: v = '\xf0'; break;
+	    case 0x144: v = '\xf1'; break;
+	    case 0x148: v = '\xf2'; break;
+	    case 0x151: v = '\xf5'; break;
+	    case 0x159: v = '\xf8'; break;
+	    case 0x16f: v = '\xf9'; break;
+	    case 0x171: v = '\xfb'; break;
+	    case 0x163: v = '\xfe'; break; /* tcedil */
+	    case 0x2d9: v = '\xff'; break;
+	 }
+	 if (v == 0) break;
+	 value = v;
+      }
+      *p = value;
+      return 1;
    case CHARSET_ISO_8859_15:
       switch (value) {
        case 0xa4: case 0xa6: case 0xb0: case 0xc4:
@@ -410,10 +492,12 @@ add_unicode(int charset, unsigned char *p, int value)
        case 0x153: value = 0xd5; break; /* &oelig; */
 #if 0
        case 0x0: value = 0xa4; break; /* euro */
-       case 0x0: value = 0xa6; break; /* Scaron */
-       case 0x0: value = 0xb0; break; /* scaron */
-       case 0x0: value = 0xc4; break; /* Zcaron */
-       case 0x0: value = 0xd0; break; /* zcaron */
+#endif
+       case 0x160: value = 0xa6; break; /* Scaron */
+       case 0x161: value = 0xb0; break; /* scaron */
+       case 0x17d: value = 0xc4; break; /* Zcaron */
+       case 0x17e: value = 0xd0; break; /* zcaron */
+#if 0
        case 0x0: value = 0xd6; break; /* Ydiersis */
 #endif
       }
@@ -442,6 +526,101 @@ add_unicode(int charset, unsigned char *p, int value)
       }
       break;
 #elif (OS==WIN32)
+   case CHARSET_WINCP1250:
+      /* MS Windows rough equivalent to ISO-8859-2 */
+      if (value >= 0x80) {
+	 int v = 0;
+	 switch (value) {
+	    case 0xa0: case 0xa4: case 0xa6: case 0xa7: case 0xa8: case 0xa9:
+	    case 0xab: case 0xac: case 0xad: case 0xae: case 0xb0: case 0xb1:
+	    case 0xb4: case 0xb5: case 0xb6: case 0xb7: case 0xb8: case 0xbb:
+	    case 0xc1: case 0xc2: case 0xc4: case 0xc7: case 0xc9: case 0xcb:
+	    case 0xcd: case 0xce: case 0xd3: case 0xd4: case 0xd6: case 0xd7:
+	    case 0xda: case 0xdc: case 0xdd: case 0xdf: case 0xe1: case 0xe2:
+	    case 0xe4: case 0xe7: case 0xe9: case 0xeb: case 0xed: case 0xee:
+	    case 0xf3: case 0xf4: case 0xf6: case 0xf7: case 0xfa: case 0xfc:
+	    case 0xfd: 
+	       v = value; break;
+	    case 0x20ac: v = '\x80'; break;
+	    case 0x201a: v = '\x82'; break;
+	    case 0x201e: v = '\x84'; break;
+	    case 0x2026: v = '\x85'; break;
+	    case 0x2020: v = '\x86'; break;
+	    case 0x2021: v = '\x87'; break;
+	    case 0x2030: v = '\x89'; break;
+	    case 0x0160: v = '\x8a'; break;
+	    case 0x2039: v = '\x8b'; break;
+	    case 0x015a: v = '\x8c'; break;
+	    case 0x0164: v = '\x8d'; break;
+	    case 0x017d: v = '\x8e'; break;
+	    case 0x0179: v = '\x8f'; break;
+	    case 0x2018: v = '\x91'; break;
+	    case 0x2019: v = '\x92'; break;
+	    case 0x201c: v = '\x93'; break;
+	    case 0x201d: v = '\x94'; break;
+	    case 0x2022: v = '\x95'; break;
+	    case 0x2013: v = '\x96'; break;
+	    case 0x2014: v = '\x97'; break;
+	    case 0x2122: v = '\x99'; break;
+	    case 0x0161: v = '\x9a'; break;
+	    case 0x203a: v = '\x9b'; break;
+	    case 0x015b: v = '\x9c'; break;
+	    case 0x0165: v = '\x9d'; break;
+	    case 0x017e: v = '\x9e'; break;
+	    case 0x017a: v = '\x9f'; break;
+	    case 0x02c7: v = '\xa1'; break;
+	    case 0x02d8: v = '\xa2'; break;
+	    case 0x0141: v = '\xa3'; break;
+	    case 0x0104: v = '\xa5'; break;
+	    case 0x015e: v = '\xaa'; break; /* Scedil */
+	    case 0x017b: v = '\xaf'; break;
+	    case 0x02db: v = '\xb2'; break;
+	    case 0x0142: v = '\xb3'; break;
+	    case 0x0105: v = '\xb9'; break;
+	    case 0x015f: v = '\xba'; break; /* scedil */
+	    case 0x013d: v = '\xbc'; break;
+	    case 0x02dd: v = '\xbd'; break;
+	    case 0x013e: v = '\xbe'; break;
+	    case 0x017c: v = '\xbf'; break;
+	    case 0x0154: v = '\xc0'; break;
+	    case 0x0102: v = '\xc3'; break;
+	    case 0x0139: v = '\xc5'; break;
+	    case 0x0106: v = '\xc6'; break;
+	    case 0x010c: v = '\xc8'; break;
+	    case 0x0118: v = '\xca'; break;
+	    case 0x011a: v = '\xcc'; break;
+	    case 0x010e: v = '\xcf'; break;
+	    case 0x0110: v = '\xd0'; break;
+	    case 0x0143: v = '\xd1'; break;
+	    case 0x0147: v = '\xd2'; break;
+	    case 0x0150: v = '\xd5'; break;
+	    case 0x0158: v = '\xd8'; break;
+	    case 0x016e: v = '\xd9'; break;
+	    case 0x0170: v = '\xdb'; break;
+	    case 0x0162: v = '\xde'; break; /* &Tcedil; */
+	    case 0x0155: v = '\xe0'; break;
+	    case 0x0103: v = '\xe3'; break;
+	    case 0x013a: v = '\xe5'; break;
+	    case 0x0107: v = '\xe6'; break;
+	    case 0x010d: v = '\xe8'; break;
+	    case 0x0119: v = '\xea'; break;
+	    case 0x011b: v = '\xec'; break;
+	    case 0x010f: v = '\xef'; break;
+	    case 0x0111: v = '\xf0'; break;
+	    case 0x0144: v = '\xf1'; break;
+	    case 0x0148: v = '\xf2'; break;
+	    case 0x0151: v = '\xf5'; break;
+	    case 0x0159: v = '\xf8'; break;
+	    case 0x016f: v = '\xf9'; break;
+	    case 0x0171: v = '\xfb'; break;
+	    case 0x0163: v = '\xfe'; break; /* tcedil */
+	    case 0x02d9: v = '\xff'; break;
+	 }
+	 if (v == 0) break;
+	 value = v;
+      }
+      *p = value;
+      return 1;
    case CHARSET_WINCP1252:
       /* MS Windows extensions to ISO-8859-1 */
       switch (value) {
@@ -572,42 +751,70 @@ add_unicode(int charset, unsigned char *p, int value)
       p[1] = 'E'; return 2;
     case 198 /*  */:
       *p = 'A'; p[1] = 'E'; return 2;
-    case 199 /*  */:
+    case 199 /*  */: case 268: /* &Ccaron; */
       *p = 'C'; return 1;
+    case 270: /* &Dcaron; */
+      *p = 'D'; return 1;
     case 200 /*  */: case 201 /*  */: case 202 /*  */: case 203 /*  */:
       *p = 'E'; return 1;
     case 204 /*  */: case 205 /*  */: case 206 /*  */: case 207 /*  */:
       *p = 'I'; return 1;
     case 208 /*  */: case 222 /*  */:
       *p = 'T'; p[1] = 'H'; return 2;
+    case 315: /* &Lacute; */
+    case 317: /* &Lcaron; */
+      *p = 'L'; return 1;
     case 209 /*  */:
       *p = 'N'; return 1;
     case 210 /*  */: case 211 /*  */: case 212 /*  */: case 213 /*  */:
       *p = 'O'; return 1;
     case 214 /*  */: /* &Ouml; */ case 0x152: /* &OElig; */
       *p = 'O'; p[1] = 'E'; return 2;
+    case 352: /* &Scaron; */
+    case 0x15e: /* &Scedil; */
+      *p = 'S'; return 1;
+    case 0x162: /* &Tcedil; */
+    case 0x164: /* &Tcaron; */
+      *p = 'T'; return 1;
     case 217 /*  */: case 218 /*  */: case 219 /*  */:
       *p = 'U'; return 1;
     case 220 /*  */: /* &Uuml; */
       *p = 'U'; p[1] = 'E'; return 2;
     case 221 /*  */:
       *p = 'Y'; return 1;
+    case 381: /* &Zcaron; */
+      *p = 'Z'; return 1;
     case 223 /*  */:
       p[1] = *p = 's'; return 2;
     case 224 /*  */: case 225 /*  */: case 226 /*  */: case 227 /*  */:
+    case 259: /* &abreve; */
       *p = 'a'; return 1;
     case 228 /*  */: /* &auml; */ case 230 /*  */:
       *p = 'a'; p[1] = 'e'; return 2;
     case 229 /*  */:
       p[1] = *p = 'a'; return 2;
-    case 231 /*  */:
+    case 231 /*  */: case 269 /* &ccaron; */:
       *p = 'c'; return 1;
+    case 271: /* &dcaron; */
+      *p = 'd'; return 1;
     case 232 /*  */: case 233 /*  */: case 234 /*  */: case 235 /*  */:
+    case 283 /* &ecaron; */:
       *p = 'e'; return 1;
     case 236 /*  */: case 237 /*  */: case 238 /*  */: case 239 /*  */:
       *p = 'i'; return 1;
-    case 241 /*  */:
+    case 316 /* &lacute; */:
+    case 318 /* &lcaron; */:
+      *p = 'l'; return 1;
+    case 241 /*  */: case 328 /* &ncaron; */:
       *p = 'n'; return 1;
+    case 345: /* &rcaron; */
+      *p = 'r'; return 1;
+    case 353: /* &scaron; */
+    case 0x15f: /* &scedil; */
+      *p = 's'; return 1;
+    case 357: /* &tcaron; */
+    case 0x163: /* &tcedil; */
+      *p = 't'; return 1;
     case 240 /*  */: case 254 /*  */:
       *p = 't'; p[1] = 'h'; return 2;
     case 242 /*  */: case 243 /*  */: case 244 /*  */: case 245 /*  */:
@@ -615,11 +822,14 @@ add_unicode(int charset, unsigned char *p, int value)
     case 246 /*  */: /* &ouml; */ case 0x153: /* &oelig; */
       *p = 'o'; p[1] = 'e'; return 2;
     case 249 /*  */: case 250 /*  */: case 251 /*  */:
+    case 367 /* &uring; */:
       *p = 'u'; return 1;
     case 252 /*  */: /* &uuml; */
       *p = 'u'; p[1] = 'e'; return 2;
     case 253 /*  */: case 255 /*  */:
       *p = 'y'; return 1;
+    case 382: /* &zcaron; */
+      *p = 'z'; return 1;
    }
 #ifdef DEBUG
    fprintf(stderr, "failed to transliterate\n");
@@ -627,8 +837,13 @@ add_unicode(int charset, unsigned char *p, int value)
    return 0;
 }
 
-/* fall back on looking in the current directory */
+#if (OS==UNIX) && defined(DATADIR) && defined(PACKAGE)
+/* Under Unix, we compile in the configured path */
+static const char *pth_cfg_files = DATADIR "/" PACKAGE;
+#else
+/* On other platforms, we fall back on looking in the current directory */
 static const char *pth_cfg_files = "";
+#endif
 
 static int num_msgs = 0;
 static char **msg_array = NULL;
@@ -794,7 +1009,7 @@ void
 msg_init(char * const *argv)
 {
    char *p;
-   ASSERT(argv);
+   SVX_ASSERT(argv);
 
 #ifdef HAVE_SIGNAL
    init_signals();
@@ -807,33 +1022,88 @@ msg_init(char * const *argv)
    appname_copy = osstrdup(argv[0]);
 #else
    /* use the lower-cased leafname on other platforms */
-   appname_copy = p = leaf_from_fnm(argv[0]);
+   p = leaf_from_fnm(argv[0]);
+   appname_copy = p;
    while (*p) {
       *p = tolower(*p);
-      p++;
+      ++p;
    }
 #endif
 
-   /* shortcut --version so you can check the version number when the correct
-    * message file can't be found... */
+   /* shortcut --version so you can check the version number even when the
+    * correct message file can't be found... */
    if (argv[1] && strcmp(argv[1], "--version") == 0) {
       cmdline_version();
       exit(0);
    }
-
-   /* Look for env. var. "SURVEXHOME" or the like */
-   p = getenv("SURVEXHOME");
-   if (p && *p) {
-      pth_cfg_files = osstrdup(p);
-      
-   /* MacOSX version can be installed anywhere, so use argv[0] */
-#if (OS==UNIX) && defined(DATADIR) && defined(PACKAGE) && !defined(__APPLE__)
-   } else {
-      /* under Unix, we compile in the configured path */
-      pth_cfg_files = DATADIR "/" PACKAGE;
+   if (argv[0]) {
+#ifdef MACOSX_BUNDLE
+      /* If we're being built into a bundle, always look relative to
+       * the path to the binary. */
+      char * pth = path_from_fnm(argv[0]);
+      pth_cfg_files = use_path(pth, "share/survex");
+      osfree(pth);
+#elif (OS==UNIX) && defined(DATADIR) && defined(PACKAGE)
+      bool free_pth = fFalse;
+      char *pth = getenv("srcdir");
+      if (!pth || !pth[0]) {
+	 pth = path_from_fnm(argv[0]);
+	 free_pth = fTrue;
+      }
+      if (pth[0]) {
+	 struct stat buf;
+#if defined(__GNUC__) && defined(__APPLE_CC__)
+	 /* On MacOS X the programs may be installed anywhere, with the
+	  * share directory and the binaries in the same directory. */
+	 p = use_path(pth, "share/survex/en.msg");
+	 if (lstat(p, &buf) == 0 && S_ISREG(buf.st_mode)) {
+	    pth_cfg_files = use_path(pth, "share/survex");
+	    goto macosx_got_msg;
+	 }
+	 osfree(p);
+#endif
+	 /* If we're run with an explicit path, check if "../lib/en.msg"
+	  * from the program's path exists, and if so look there for
+	  * support files - this allows us to test binaries in the build
+	  * tree easily. */
+	 p = use_path(pth, "../lib/en.msg");
+	 if (lstat(p, &buf) == 0) {
+#ifdef S_ISDIR
+	    /* POSIX way */
+	    if (S_ISREG(buf.st_mode)) {
+	       pth_cfg_files = use_path(pth, "../lib");
+	    }
 #else
-   } else if (argv[0]) {
-      /* else try the path on argv[0] */
+	    /* BSD way */
+	    if ((buf.st_mode & S_IFMT) == S_IFREG) {
+	       pth_cfg_files = use_path(pth, "../lib");
+	    }
+#endif
+	 }
+#if defined(__GNUC__) && defined(__APPLE_CC__)
+macosx_got_msg:
+#endif
+	 osfree(p);
+      }
+      
+      if (free_pth) osfree(pth);
+#elif (OS==WIN32)
+      DWORD len = 256;
+      char *buf = NULL, *modname;
+      while (1) {
+	  DWORD got;
+	  buf = osrealloc(buf, len);
+	  got = GetModuleFileName(NULL, buf, len);
+	  if (got < len) break;
+	  len += len;
+      }
+      modname = buf;
+      /* Strange Win32 nastiness - strip prefix "\\?\" if present */
+      if (strncmp(modname, "\\\\?\\", 4) == 0) modname += 4;
+      pth_cfg_files = path_from_fnm(modname);
+      osfree(buf);
+#else
+      /* Get the path to the support files from argv[0] */
       pth_cfg_files = path_from_fnm(argv[0]);
 #endif
    }
@@ -868,6 +1138,9 @@ msg_init(char * const *argv)
 	     case LANG_CATALAN:
 	       msg_lang = "ca";
 	       break;
+	     case LANG_CHINESE:
+	       msg_lang = "zh";
+	       break;
 	     case LANG_ENGLISH:
 	       if (SUBLANGID(langid) == SUBLANG_ENGLISH_US)
 		  msg_lang = "en_US";
@@ -898,6 +1171,12 @@ msg_init(char * const *argv)
 	       else
 		  msg_lang = "pt";
 	       break;
+	     case LANG_ROMANIAN:
+	       msg_lang = "ro";
+	       break;
+	     case LANG_SLOVAK:
+	       msg_lang = "sk";
+	       break;
 	     case LANG_SPANISH:
 	       msg_lang = "es";
 	       break;
@@ -931,6 +1210,9 @@ msg_init(char * const *argv)
 	  case 8: /* Portugal */
 	    msg_lang = "pt";
 	    break;
+	  case 23: /* Hong Kong */
+	    msg_lang = "zh";
+	    break;
 	  case 48: /* USA */
 	    msg_lang = "en_US";
 	    break;
@@ -945,7 +1227,6 @@ msg_init(char * const *argv)
 	  case 16: /* Iceland */
 	  case 20: /* Turkey */
 	  case 21: /* Arabic */
-	  case 23: /* Hong Kong */
 	  case 24: /* Russia */
 	  case 25: /* Russia2 */
 	  case 26: /* Israel */
@@ -965,9 +1246,9 @@ msg_init(char * const *argv)
 	      r.x.cx = 2048;
 	      /* bit 1 is the carry flag */
 	      if (__dpmi_int(0x21, &r) != -1 && !(r.x.flags & 1)) {
-		 unsigned short p;
-		 dosmemget(__tb + 3, 2, &p);
-		 country_code = p;
+		 unsigned short val;
+		 dosmemget(__tb + 3, 2, &val);
+		 country_code = val;
 # else
 	      union REGS r;
 	      r.x.ax = 0x3800; /* get current country info */
@@ -1120,6 +1401,19 @@ msg_init(char * const *argv)
 		     case 351: /* Portugal */
 			 msg_lang = "pt";
 			 break;
+		     case 42: /* Czechoslovakia / Tjekia / Slovakia (not supported by DR DOS 5.0) */
+		     case 421: /* Czech Republic / Tjekia (PC DOS 7+) */
+		     case 422: /* Slovakia (reported as 421 due to a bug in COUNTRY.SYS) */
+			 msg_lang = "sk";
+			 break;
+		     case 65: /* Singapore */
+		     case 86: /* China (MS-DOS 5.0+) */
+		     case 88: /* Taiwan (MS-DOS 5.0+) */
+		     case 852: /* Hong Kong */
+		     case 853: /* Macao */
+		     case 886: /* Taiwan (MS-DOS 6.22+) */
+			 msg_lang = "zh";
+			 break;
 #if 0
 		     case 7: /* Russia */
 		     case 20: /* Egypt */
@@ -1128,21 +1422,21 @@ msg_init(char * const *argv)
 		     case 35: /* Bulgaria??? */
 		     case 36: /* Hungary (not supported by DR DOS 5.0) */
 		     case 38: /* Yugoslavia (not supported by DR DOS 5.0) -- obsolete */
+#endif
 		     case 40: /* Romania */
-		     case 42: /* Czechoslovakia / Tjekia / Slovakia (not supported by DR DOS 5.0) */
+			 msg_lang = "ro";
+			 break;
+#if 0
 		     case 45: /* Denmark */
 		     case 46: /* Sweden */
 		     case 47: /* Norway */
 		     case 48: /* Poland (not supported by DR DOS 5.0) */
 		     case 60: /* Malaysia */
 		     case 62: /* Indonesia / East Timor */
-		     case 65: /* Singapore */
 		     case 66: /* Thailand (or Taiwan??? ) */
 		     case 81: /* Japan (DR DOS 5.0, MS-DOS 5.0+) */
 		     case 82: /* South Korea (DR DOS 5.0) */
 		     case 84: /* Vietnam */
-		     case 86: /* China (MS-DOS 5.0+) */
-		     case 88: /* Taiwan (MS-DOS 5.0+) */
 		     case 90: /* Turkey (MS-DOS 5.0+) */
 		     case 91: /* India */
 		     case 92: /* Pakistan */
@@ -1178,7 +1472,7 @@ msg_init(char * const *argv)
 		     case 370: /* Lithuania (reported as 372 due to a bug in MS-DOS COUNTRY.SYS) */
 		     case 371: /* Latvia (reported as 372 due to a bug in MS-DOS COUNTRY.SYS) */
 		     case 372: /* Estonia */
-		     case 373: /* Moldova */
+		     case 373: /* Moldova */ /* FIXME: similar to Romanian? */
 		     case 375: /* ??? (MS-DOS 7.10 / Windows98) */
 		     case 380: /* Ukraine */
 		     case 381: /* Serbia / Montenegro */
@@ -1188,8 +1482,6 @@ msg_init(char * const *argv)
 		     case 387: /* Bosnia-Herzegovina (Latin) */
 		     case 388: /* Bosnia-Herzegovina (Cyrillic) (PC DOS 7+) (reported as 381 due to a bug in PC DOS COUNTRY.SYS) */
 		     case 389: /* FYR Macedonia */
-		     case 421: /* Czech Republic / Tjekia (PC DOS 7+) */
-		     case 422: /* Slovakia (reported as 421 due to a bug in COUNTRY.SYS) */
 		     case 597: /* Suriname (nl) */
 		     case 599: /* Netherland Antilles (nl) */
 		     case 666: /* Russia??? (PTS-DOS 6.51 KEYB) */
@@ -1201,12 +1493,9 @@ msg_init(char * const *argv)
 		     case 785: /* Arabic (Middle East/Saudi Arabia/etc.) */
 		     case 804: /* Ukraine */
 		     case 850: /* North Korea */
-		     case 852: /* Hong Kong */
-		     case 853: /* Macao */
 		     case 855: /* Cambodia */
 		     case 856: /* Laos */
 		     case 880: /* Bangladesh */
-		     case 886: /* Taiwan (MS-DOS 6.22+) */
 		     case 960: /* Maldives */
 		     case 962: /* Jordan */
 		     case 963: /* Syria / Syrian Arab Republic */
@@ -1272,19 +1561,21 @@ msg_init(char * const *argv)
 const char *
 msg(int en)
 {
-   /* NB can't use ASSERT here! */
+   /* NB can't use SVX_ASSERT here! */
    static char badbuf[256];
    if (en >= 1000 && en < 1000 + N_DONTEXTRACTMSGS)
       return dontextract[en - 1000];
    if (!msg_array) {
       if (en != 1)  {
-	 sprintf(badbuf, "Message %d requested before msg_array initialised\n", en);
+	 sprintf(badbuf, "Message %d requested before msg_array initialised\n",
+		 en);
 	 return badbuf;
       }
       /* this should be the only other message which can be requested before
        * the message file is opened and read... */
       if (!dontextract) return "Out of memory (couldn't find %lu bytes).";
-      return dontextract[4];
+      return dontextract[(/*Out of memory (couldn't find %lu bytes).*/1004)
+			 - 1000];
    }
 
    if (en < 0 || en >= num_msgs) {
