@@ -61,6 +61,8 @@ class svxPrintout : public wxPrintout {
     wxColour colour_cross,colour_surface_leg;
 
     long x_t, y_t;
+    double font_scaling_x, font_scaling_y;
+    wxFont * current_font;
 
     int check_intersection(long x_p, long y_p);
     void draw_info_box();
@@ -100,107 +102,149 @@ class svxPrintout : public wxPrintout {
 
 BEGIN_EVENT_TABLE(svxPrintDlg, wxDialog)
     EVT_TEXT(svx_SCALE, svxPrintDlg::OnChange)
-    EVT_RADIOBOX(svx_ASPECT, svxPrintDlg::OnChange)
-    EVT_SPINCTRL(svx_BEARING, svxPrintDlg::OnChange)
-    EVT_SPINCTRL(svx_TILT, svxPrintDlg::OnChange)
+    EVT_COMBOBOX(svx_SCALE, svxPrintDlg::OnChange)
+    EVT_SPINCTRL(svx_BEARING, svxPrintDlg::OnChangeSpin)
+    EVT_SPINCTRL(svx_TILT, svxPrintDlg::OnChangeSpin)
     EVT_BUTTON(svx_PRINT, svxPrintDlg::OnPrint)
     EVT_BUTTON(svx_PREVIEW, svxPrintDlg::OnPreview)
+    EVT_BUTTON(svx_PLAN, svxPrintDlg::OnPlan)
+    EVT_BUTTON(svx_ELEV, svxPrintDlg::OnElevation)
 END_EVENT_TABLE()
 
+static const wxString scales[] = {
+    "One page", // FIXME TRANSLATE
+    "25",
+    "50",
+    "100",
+    "250",
+    "500",
+    "1000",
+    "2500",
+    "5000",
+    "10000",
+    "25000",
+    "50000",
+    "100000"
+};
+
 // there are three jobs to do here...
 // User <-> wx - this should possibly be done in a separate file
 svxPrintDlg::svxPrintDlg(MainFrm* parent, const wxString & filename,
 			 const wxString & title, const wxString & datestamp,
 			 double angle, double tilt_angle,
 			 bool labels, bool crosses, bool legs, bool surf)
-	: wxDialog(parent, -1, wxString("Print")),
+	: wxDialog(parent, -1, wxString(msg(/*Print*/399))),
 	  m_File(filename), m_parent(parent)
 {
-    m_layout = layout_new();
-    // FIXME rot and tilt shouldn't be integers, but for now add a small
-    // fraction before forcing to int as otherwise plan view ends up being
-    // 89 degrees!
-    m_layout->rot = int(deg(angle) + .001);
-    m_layout->tilt = int(deg(tilt_angle) + .001);
+    m_layout = new layout;
     m_layout->Labels = labels;
     m_layout->Crosses = crosses;
     m_layout->Shots = legs;
     m_layout->Surface = surf;
-    m_layout->title = osstrdup(title.c_str());
     m_layout->datestamp = osstrdup(datestamp.c_str());
+    if (title.length() > 11 &&
+	title.substr(title.length() - 11) == " (extended)") {
+	m_layout->title = osstrdup(title.substr(0, title.length() - 11).c_str());
+	m_layout->view = layout::EXTELEV;
+	m_layout->rot = int(deg(angle) + .001);
+	if (m_layout->rot != 0 && m_layout->rot != 180) m_layout->rot = 0;
+	m_layout->tilt = 0;
+    } else {
+	// FIXME rot and tilt shouldn't be integers, but for now add a small
+	// fraction before forcing to int as otherwise plan view ends up being
+	// 89 degrees!
+	m_layout->title = osstrdup(title.c_str());
+	m_layout->tilt = int(deg(tilt_angle) + .001);
+	if (m_layout->tilt == 90) {
+	    m_layout->view = layout::PLAN;
+	} else if (m_layout->tilt == 0) {
+	    m_layout->view = layout::ELEV;
+	} else {
+	    m_layout->view = layout::TILT;
+	}
+    }
 
     /* setup our print dialog*/
     wxBoxSizer* v1 = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer* h1 = new wxBoxSizer(wxHORIZONTAL); // holds controls
-    wxBoxSizer* v2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, "View"), wxVERTICAL); 
-    wxBoxSizer* v3 = new wxStaticBoxSizer(new wxStaticBox(this, -1, "Elements"), wxVERTICAL); 
+    wxBoxSizer* v2 = new wxStaticBoxSizer(new wxStaticBox(this, -1, msg(/*View*/255)), wxVERTICAL); 
+    wxBoxSizer* v3 = new wxStaticBoxSizer(new wxStaticBox(this, -1, "Elements"), wxVERTICAL); // FIXME TRANSLATE
     wxBoxSizer* h2 = new wxBoxSizer(wxHORIZONTAL); // holds buttons
 
+    { // this isn't the "too wide" bit...
+    wxStaticText* label;
+    label = new wxStaticText(this, -1, wxString(msg(/*Scale*/154)) + " 1:");
+    m_scale = new wxComboBox(this, svx_SCALE, scales[0], wxDefaultPosition,
+			     wxDefaultSize, sizeof(scales) / sizeof(scales[0]),
+			     scales);
     wxBoxSizer* scalebox = new wxBoxSizer(wxHORIZONTAL);
-    scalebox->Add(new wxStaticText(this, -1,
-				   wxString(msg(/*Scale*/154)) + " 1:"),
-		  0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
-    wxString choices[] = {
-	    "25",
-	    "50",
-	    "100",
-	    "250",
-	    "500",
-	    "1000",
-	    "2500",
-	    "5000",
-	    "10000",
-	    "25000",
-	    "50000",
-	    "100000"};
-    m_scale = new wxComboBox(this,svx_SCALE,"500",wxDefaultPosition,wxDefaultSize,12,choices);
-    scalebox->Add(m_scale,1,wxALIGN_LEFT + wxALIGN_CENTER_VERTICAL + wxALL,5);
-    v2->Add(scalebox, 0, wxALIGN_RIGHT + wxALL, 0);
-    // Make the dummy string wider than any sane value so the sizer
-    // picks a suitable width...
-    m_printSize = new wxStaticText(this, -1, "9604 Pages (98x98)");
-    v2->Add(m_printSize, 0, wxALIGN_LEFT + wxALL, 5);
-    static const wxString radio_choices[] = { "Plan", "Elevation", "Tilted" };
-    m_aspect = new wxRadioBox(this, svx_ASPECT, "Orientation",
-			      wxDefaultPosition, wxDefaultSize, 3, 
-			      radio_choices);
-    v2->Add(m_aspect, 0, wxALIGN_LEFT + wxALL, 5);
+    scalebox->Add(label, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+    scalebox->Add(m_scale, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
+
+    v2->Add(scalebox, 0, wxALIGN_LEFT|wxALL, 0);
+    }
+
+    // Make the dummy string wider than any sane value and use that to
+    // fix the width of the control so the sizers allow space for bigger
+    // page layouts.
+    m_printSize = new wxStaticText(this, -1, wxString::Format(msg(/*%d pages (%dx%d)*/257), 9604, 98, 98), wxDefaultPosition, wxDefaultSize, wxST_NO_AUTORESIZE);
+    v2->Add(m_printSize, 0, wxALIGN_LEFT|wxALL, 5);
+
+    { // this isn't the "too wide" bit...
     wxFlexGridSizer* anglebox = new wxFlexGridSizer(2);
-    m_tilttext = new wxStaticText(this, -1, "Tilt angle");
-    anglebox->Add(m_tilttext,0,wxALIGN_CENTER_VERTICAL + wxALIGN_RIGHT + wxALL,5);
+    wxStaticText * brg_label;
+    brg_label = new wxStaticText(this, -1, msg(/*Bearing*/259));
+    anglebox->Add(brg_label, 0, wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT|wxALL, 5);
+    m_bearing = new wxSpinCtrl(this, svx_BEARING);
+    m_bearing->SetRange(0, 359);
+    anglebox->Add(m_bearing, 0, wxALIGN_CENTER|wxALL, 5);
+    m_tilttext = new wxStaticText(this, -1, "Tilt angle"); // FIXME TRANSLATE
+    anglebox->Add(m_tilttext, 0, wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT|wxALL, 5);
     m_tilt = new wxSpinCtrl(this,svx_TILT);
-    anglebox->Add(m_tilt,0,wxALIGN_CENTER_VERTICAL + wxALIGN_LEFT + wxALL,5);
-    m_tilt->SetRange(-90,90);
-    anglebox->Add(new wxStaticText(this,-1,"Bearing"),0,wxALIGN_CENTER_VERTICAL + wxALIGN_RIGHT + wxALL,5);
-    m_bearing = new wxSpinCtrl(this,svx_BEARING);
-    anglebox->Add(m_bearing,0,wxALIGN_CENTER_VERTICAL + wxALIGN_LEFT + wxALL,5);
-    m_bearing->SetRange(0,359);
-    v2->Add(anglebox,0,wxALIGN_LEFT + wxALL,0);
-    h1->Add(v2,0,wxALIGN_LEFT + wxALL,5);
+    m_tilt->SetRange(-90, 90);
+    anglebox->Add(m_tilt, 0, wxALIGN_CENTER|wxALL, 5);
+
+    v2->Add(anglebox, 0, wxALIGN_LEFT|wxALL, 0);
+    }
     
-    m_legs = new wxCheckBox(this, svx_LEGS, "Legs");
+    if (m_layout->view != layout::EXTELEV) {
+	wxBoxSizer * planelevsizer = new wxBoxSizer(wxHORIZONTAL);
+	planelevsizer->Add(new wxButton(this, svx_PLAN, "Plan"),
+			   0, wxALIGN_CENTRE_VERTICAL|wxALL, 5);
+	planelevsizer->Add(new wxButton(this, svx_ELEV, "Elevation"),
+			   0, wxALIGN_CENTRE_VERTICAL|wxALL, 5);
+
+	v2->Add(planelevsizer, 0, wxALIGN_LEFT|wxALL, 5);
+    }
+
+    h1->Add(v2, 0, wxALIGN_LEFT|wxALL, 5);
+
+    m_legs = new wxCheckBox(this, svx_LEGS, msg(/*Underground Survey Legs*/262));
     v3->Add(m_legs, 0, wxALIGN_LEFT|wxALL, 2);
-    m_surface = new wxCheckBox(this, svx_SCALEBAR, msg(/*Surface survey legs*/403));
+    m_surface = new wxCheckBox(this, svx_SCALEBAR, msg(/*Sur&amp;face Survey Legs*/403));
     v3->Add(m_surface, 0, wxALIGN_LEFT|wxALL, 2);
-    m_stations = new wxCheckBox(this, svx_STATIONS, "Stations");
+    m_stations = new wxCheckBox(this, svx_STATIONS, msg(/*Crosses*/261));
     v3->Add(m_stations, 0, wxALIGN_LEFT|wxALL, 2);
-    m_names = new wxCheckBox(this, svx_NAMES, "Station names");
+    m_names = new wxCheckBox(this, svx_NAMES, msg(/*Station Names*/260));
     v3->Add(m_names, 0, wxALIGN_LEFT|wxALL, 2);
-    m_borders = new wxCheckBox(this, svx_BORDERS, "Borders");
+    m_borders = new wxCheckBox(this, svx_BORDERS, "Borders"); // FIXME TRANSLATE
     v3->Add(m_borders, 0, wxALIGN_LEFT|wxALL, 2);
 //    m_blanks = new wxCheckBox(this, svx_BLANKS, "Blank Pages");
 //    v3->Add(m_blanks, 0, wxALIGN_LEFT|wxALL, 2);
-    m_infoBox = new wxCheckBox(this, svx_INFOBOX, "Info Box");
+    m_infoBox = new wxCheckBox(this, svx_INFOBOX, "Info Box"); // FIXME TRANSLATE
     v3->Add(m_infoBox, 0, wxALIGN_LEFT|wxALL, 2);
+    
     h1->Add(v3, 0, wxALIGN_LEFT|wxALL, 5);
+
     v1->Add(h1, 0, wxALIGN_LEFT|wxALL, 5);
 
     wxButton * but;
-    but = new wxButton(this, svx_PRINT, msg(/*&Print*/400));
+    but = new wxButton(this, wxID_CANCEL, msg(/*&Cancel*/402));
     h2->Add(but, 0, wxALIGN_RIGHT|wxALL, 5);
     but = new wxButton(this, svx_PREVIEW, msg(/*Pre&view*/401));
     h2->Add(but, 0, wxALIGN_RIGHT|wxALL, 5);
-    but = new wxButton(this, wxID_CANCEL, msg(/*&Cancel*/402));
+    but = new wxButton(this, svx_PRINT, msg(/*&Print*/400));
+    but->SetDefault();
     h2->Add(but, 0, wxALIGN_RIGHT|wxALL, 5);
     v1->Add(h2, 0, wxALIGN_RIGHT|wxALL, 5);
     
@@ -211,31 +255,31 @@ svxPrintDlg::svxPrintDlg(MainFrm* parent, const wxString & filename,
     v1->Fit(this);
     v1->SetSizeHints(this);
 
-    wxCommandEvent dummy;
-    OnChange(dummy);
+    SomethingChanged();
 }
 
 svxPrintDlg::~svxPrintDlg() {
     osfree(m_layout->title);
     osfree(m_layout->datestamp);
-    layout_destroy(m_layout);
+    delete m_layout;
     m_layout = NULL;
 }
 
 void 
-svxPrintDlg::OnPrint(wxCommandEvent& event) {
+svxPrintDlg::OnPrint(wxCommandEvent&) {
+    SomethingChanged();
     wxPrintDialogData pd(m_parent->GetPageSetupData()->GetPrintData());
     wxPrinter pr(&pd);
     svxPrintout po(m_parent, m_layout, m_parent->GetPageSetupData(), m_File);
     if (pr.Print(this, &po, true)) {
 	// Close the print dialog if printing succeeded.
-	EndModal(wxOK);
+	Destroy();
     }
 }
 
 void 
-svxPrintDlg::OnPreview(wxCommandEvent& event) {
-    OnChange(event);
+svxPrintDlg::OnPreview(wxCommandEvent&) {
+    SomethingChanged();
     wxPrintDialogData pd(m_parent->GetPageSetupData()->GetPrintData());
     wxPrintPreview* pv;
     pv = new wxPrintPreview(new svxPrintout(m_parent, m_layout,
@@ -243,47 +287,78 @@ svxPrintDlg::OnPreview(wxCommandEvent& event) {
 			    new svxPrintout(m_parent, m_layout,
 					    m_parent->GetPageSetupData(), m_File),
 			    &pd);
-    wxPreviewFrame *frame = new wxPreviewFrame(pv, m_parent, "Print Preview");
-    frame->Centre(wxBOTH);
+    wxPreviewFrame *frame = new wxPreviewFrame(pv, m_parent, msg(/*Print Preview*/398));
     frame->Initialize();
+
+    // Size preview frame so that all of the controlbar and canvas can be seen
+    // if possible.
     int w, h;
-    frame->GetSize(&w, &h);
+    // GetBestSize gives us the width needed to show the whole controlbar.
+    frame->GetBestSize(&w, &h);
 #ifdef __WXMAC__
     // wxMac opens the preview window at minimum size by default.
     // 360x480 is apparently enough to show A4 portrait.
     if (h < 480 || w < 360) {
 	if (h < 480) h = 480;
 	if (w < 360) w = 360;
-	frame->SetSize(w, h);
     }
 #else
     if (h < w) {
-	// By default the preview window opens wide enough, but not anywhere
-	// near tall enough (on wxGTK anyway).
+	// On wxGTK at least, GetBestSize() returns much too small a height.
 	h = w * 6 / 5;
-	frame->SetSize(w, h);
     }
 #endif
-    frame->Show(TRUE);
+    // Ensure that we don't make the window bigger than the screen.
+    // Use wxGetClientDisplayRect() so we don't cover the MS Windows
+    // task bar either.
+    wxRect disp = wxGetClientDisplayRect();
+    if (w > disp.GetWidth()) w = disp.GetWidth();
+    if (h > disp.GetHeight()) h = disp.GetHeight();
+    // Centre the window within the "ClientDisplayRect".
+    int x = disp.GetLeft() + (disp.GetWidth() - w) / 2;
+    int y = disp.GetTop() + (disp.GetHeight() - h) / 2;
+    frame->SetSize(x, y, w, h);
+
+    frame->Show();
 }
 
 void 
-svxPrintDlg::OnChange(wxCommandEvent& event) {
-    wxPageSetupDialogData* data = m_parent->GetPageSetupData();
+svxPrintDlg::OnPlan(wxCommandEvent&) {
+    m_tilt->SetValue(90);
+    SomethingChanged();
+}
+
+void 
+svxPrintDlg::OnElevation(wxCommandEvent&) {
+    m_tilt->SetValue(0);
+    SomethingChanged();
+}
+
+void 
+svxPrintDlg::OnChangeSpin(wxSpinEvent&e) {
+    SomethingChanged();
+}
+
+void 
+svxPrintDlg::OnChange(wxCommandEvent&) {
+    SomethingChanged();
+}
+
+void
+svxPrintDlg::SomethingChanged() {
     UIToLayout();
     // Update the bounding box.
     RecalcBounds();
     if (m_layout->xMax >= m_layout->xMin) { 
+	wxPageSetupDialogData* data = m_parent->GetPageSetupData();
 	// get print details...
 	m_layout->PaperWidth = data->GetPaperSize().GetWidth() - 
 	    data->GetMarginBottomRight().x - data->GetMarginTopLeft().x;
 	m_layout->PaperDepth = data->GetPaperSize().GetHeight() - 
 	    data->GetMarginBottomRight().y - data->GetMarginTopLeft().y;
-	pages_required(m_layout);
+	m_layout->pages_required();
 	m_layout->pages = m_layout->pagesX * m_layout->pagesY;
-	wxString temp;
-	temp.Printf("%d pages (%d x %d)",m_layout->pages,m_layout->pagesX,m_layout->pagesY);
-	m_printSize->SetLabel(temp);
+	m_printSize->SetLabel(wxString::Format(msg(/*%d pages (%dx%d)*/257), m_layout->pages, m_layout->pagesX, m_layout->pagesY));
     }
 }
 
@@ -297,21 +372,23 @@ svxPrintDlg::LayoutToUI(){
     m_infoBox->SetValue(!m_layout->Raw);
     m_surface->SetValue(m_layout->Surface);
     m_tilt->SetValue(m_layout->tilt);
+    // FIXME is EXTELEV disable both buttons and tilt and rot spinctrls
+    // FIXME: enable both buttons
     if (m_layout->tilt > 89) {
-	m_aspect->SetSelection(0);
-	m_tilt->Disable();
+	// FIXME: disable Plan button
     } else if (m_layout->tilt == 0) {
-	m_aspect->SetSelection(1);
-	m_tilt->Disable();
-    } else {
-	m_aspect->SetSelection(2);
-	m_tilt->Enable();
+	// FIXME: disable Elevation button
     }
+
     m_bearing->SetValue(m_layout->rot);
     // Do this last as it causes an OnChange message which calls UIToLayout
-    wxString temp;
-    temp << m_layout->Scale;
-    m_scale->SetValue(temp);
+    if (m_layout->Scale != 0) {
+	wxString temp;
+	temp << m_layout->Scale;
+	m_scale->SetValue(temp);
+    } else {
+	m_scale->SetValue(scales[0]);
+    }
 }
 
 void 
@@ -324,29 +401,28 @@ svxPrintDlg::UIToLayout(){
     m_layout->Raw = !m_infoBox->IsChecked();
     m_layout->Surface = m_surface->IsChecked();
 
+    if (m_layout->view != layout::EXTELEV) {
+	m_layout->tilt = m_tilt->GetValue();
+	if (m_layout->tilt == 90) {
+	    m_layout->view = layout::PLAN;
+	} else if (m_layout->tilt == 0) {
+	    m_layout->view = layout::ELEV;
+	} else {
+	    m_layout->view = layout::TILT;
+	}
+	m_layout->rot = m_bearing->GetValue();
+    }
+
     (m_scale->GetValue()).ToDouble(&(m_layout->Scale));
-    m_layout->Sc = 1000 / m_layout->Scale;
+    if (m_layout->Scale == 0.0) {
+	double x = 1000.0 / m_layout->pick_scale(1, 1);
+
+	/* trim to 2 s.f. (rounding up) */
+	double w = pow(10.0, floor(log10(x) - 1.0));
+	x = ceil(x / w) * w;
 
-    switch(m_aspect->GetSelection()) {
-	case 0: // Plan;
-	    m_layout->tilt = 90;
-	    m_tilt->SetValue(90);
-	    m_tilt->Disable();
-	    m_tilttext->Disable();
-	    break;
-	case 1: // Elevation;
-	    m_layout->tilt = 0;
-	    m_tilt->SetValue(0);
-	    m_tilt->Disable();
-	    m_tilttext->Disable();
-	    break;
-	case 2: // Tilted
-	    m_layout->tilt = m_tilt->GetValue();
-	    m_tilt->Enable();
-	    m_tilttext->Enable();
-	    break;
+	m_layout->Scale = x;
     }
-    m_layout->rot = m_bearing->GetValue();
 }
 
 void
@@ -399,11 +475,9 @@ svxPrintDlg::RecalcBounds()
     }
 }
 
-static enum {PLAN, ELEV, TILT, EXTELEV} view = PLAN;
-
 #define DEG "\xB0" /* degree symbol in iso-8859-1 */
 
-static long xpPageWidth, ypPageDepth;
+static int xpPageWidth, ypPageDepth;
 static long MarginLeft, MarginRight, MarginTop, MarginBottom;
 static long x_offset,y_offset;
 static wxFont *font_labels, *font_default;
@@ -413,7 +487,6 @@ static int fontsize, fontsize_labels;
 
 static const char *fontname = "Arial", *fontname_labels = "Arial";
 
-
 // wx <-> prcore (calls to print_page etc...)
 svxPrintout::svxPrintout(MainFrm *mainfrm, layout *l, wxPageSetupData *data,
 			 const wxString & title)
@@ -436,7 +509,7 @@ svxPrintout::draw_info_box()
 
    SetColour(PR_COLOUR_FRAME);
 
-   if (view != EXTELEV) {
+   if (l->view != layout::EXTELEV) {
       boxwidth = 100;
       boxheight = 40;
       MOVEMM(60,40);
@@ -455,8 +528,8 @@ svxPrintout::draw_info_box()
    MOVEMM(0, 20); DRAWMM(60, 20);
    MOVEMM(0, 10); DRAWMM(60, 10);
 
-   switch (view) {
-    case PLAN: {
+   switch (l->view) {
+    case layout::PLAN: {
       long ax, ay, bx, by, cx, cy, dx, dy;
 
 #define RADIUS 16.0
@@ -487,7 +560,7 @@ svxPrintout::draw_info_box()
       WriteString(msg(/*Plan view*/117));
       break;
     }
-    case ELEV: case TILT:
+    case layout::ELEV: case layout::TILT:
       MOVEMM(65, 15); DRAWMM(70, 12); DRAWMM(68, 15); DRAWMM(70, 18);
 
       DRAWMM(65, 15); DRAWMM(95, 15);
@@ -508,7 +581,7 @@ svxPrintout::draw_info_box()
       MOVEMM(5, 23);
       WriteString(msg(/*Elevation*/118));
       break;
-    case EXTELEV:
+    case layout::EXTELEV:
       SetColour(PR_COLOUR_TEXT);
       MOVEMM(5, 13);
       WriteString(msg(/*Extended elevation*/191));
@@ -522,8 +595,9 @@ svxPrintout::draw_info_box()
    sprintf(p, " 1:%.0f", l->Scale);
    MOVEMM(5, boxheight - 27); WriteString(szTmp);
 
-   if (view != EXTELEV) {
-      strcpy(szTmp, msg(view == PLAN ? /*Up page*/168 : /*View*/169));
+   if (l->view != layout::EXTELEV) {
+      strcpy(szTmp,
+	     msg(l->view == layout::PLAN ? /*Up page*/168 : /*View*/169));
       p = szTmp + strlen(szTmp);
       sprintf(p, " %03d"DEG, l->rot);
       MOVEMM(5, 3); WriteString(szTmp);
@@ -650,7 +724,7 @@ make_calibration(layout *l) {
       pt.x = 0.001;
       pt.y = 0.05;
       stack(l,img_LABEL, "10cm", &pt);
-      l->Sc = 1000.0;
+      l->Scale = 1.0;
 }
 #endif
 
@@ -821,24 +895,18 @@ svxPrintout::drawticks(border clip, int tsize, int x, int y)
 bool 
 svxPrintout::OnPrintPage(int pageNum) {
     pdc = GetDC();
-    int w, h;
-    GetPageSizePixels(&w, &h);
-    if (IsPreview()) {
-	wxSize sz = pdc->GetSize();
-	xpPageWidth = sz.GetWidth();
-	ypPageDepth = sz.GetHeight();
-	font_labels->SetPointSize((int)(fontsize_labels * xpPageWidth / w));
-	font_default->SetPointSize((int)(fontsize * xpPageWidth / w));
-    } else {
-	font_labels->SetPointSize(fontsize_labels);
-	font_default->SetPointSize(fontsize);
-	xpPageWidth = w;
-	ypPageDepth = h;
-    }
+    GetPageSizePixels(&xpPageWidth, &ypPageDepth);
     int PaperWidth, PaperDepth;
     GetPageSizeMM(&PaperWidth, &PaperDepth);
     m_layout->scX = (double)xpPageWidth / PaperWidth;
     m_layout->scY = (double)ypPageDepth / PaperDepth;
+    if (IsPreview()) {
+	wxSize sz = pdc->GetSize();
+	pdc->SetUserScale((double)sz.GetWidth() / xpPageWidth,
+			  (double)sz.GetHeight() / ypPageDepth);
+    }
+    font_scaling_x = xpPageWidth / PaperWidth * 25.4 / 72.0;
+    font_scaling_y = ypPageDepth / PaperDepth * 25.4 / 72.0;
     MarginLeft = m_data->GetMarginTopLeft().x;
     MarginTop = m_data->GetMarginTopLeft().y;
     MarginBottom = m_data->GetMarginBottomRight().y;
@@ -859,7 +927,7 @@ svxPrintout::OnPrintPage(int pageNum) {
     SINT = sin(rad(l->tilt));
     COST = cos(rad(l->tilt));
 
-    long x, y;
+    long x = 0, y = 0;
     bool pending_move = false;
     bool last_leg_surface = false;
 
@@ -870,6 +938,8 @@ svxPrintout::OnPrintPage(int pageNum) {
 	draw_info_box();
     }
 
+    double Sc = 1000 / m_layout->Scale;
+
     if (l->Surface || l->Shots) {
 	for (int i=0; i < m_parent->GetNumDepthBands(); ++i) {
 	    list<PointInfo*>::const_iterator p = m_parent->GetPoints(i);
@@ -879,8 +949,8 @@ svxPrintout::OnPrintPage(int pageNum) {
 		double pz = (*p)->GetZ();
 		double X = px * COS - py * SIN;
 		double Y = (px * SIN + py * COS) * SINT + pz * COST;
-		long xnew = (long)((X * l->Sc + l->xOrg) * l->scX);
-		long ynew = (long)((Y * l->Sc + l->yOrg) * l->scY);
+		long xnew = (long)((X * Sc + l->xOrg) * l->scX);
+		long ynew = (long)((Y * Sc + l->yOrg) * l->scY);
 
 		if ((*p)->IsLine()) {
 		    bool draw = ((*p)->IsSurface() ? l->Surface : l->Shots);
@@ -929,8 +999,8 @@ svxPrintout::OnPrintPage(int pageNum) {
 		double X = px * COS - py * SIN;
 		double Y = (px * SIN + py * COS) * SINT + pz * COST;
 		long xnew, ynew;
-		xnew = (long)((X * l->Sc + l->xOrg) * l->scX);
-		ynew = (long)((Y * l->Sc + l->yOrg) * l->scY);
+		xnew = (long)((X * Sc + l->xOrg) * l->scX);
+		ynew = (long)((Y * Sc + l->yOrg) * l->scY);
 		if (l->Crosses) {
 		    SetColour(PR_COLOUR_CROSS);
 		    DrawCross(xnew, ynew);
@@ -1150,10 +1220,10 @@ svxPrintout::SetFont(int fontcode)
 {
     switch (fontcode) {
 	case PR_FONT_DEFAULT:
-	    pdc->SetFont(*font_default);
+	    current_font = font_default;
 	    break;
 	case PR_FONT_LABELS:
-	    pdc->SetFont(*font_labels);
+	    current_font = font_labels;
 	    break;
 	default:
 	    BUG("unknown font code");
@@ -1191,10 +1261,16 @@ svxPrintout::SetColour(int colourcode)
 void
 svxPrintout::WriteString(const char *s)
 {
-    int w,h;
-    pdc->GetTextExtent("My", &w, &h);
+    double xsc, ysc;
+    pdc->GetUserScale(&xsc, &ysc);
+    pdc->SetUserScale(xsc * font_scaling_x, ysc * font_scaling_y);
+    pdc->SetFont(*current_font);
+    int w, h;
     if (cur_pass != -1) {
-	pdc->DrawText(s, x_t, y_t - h);
+	pdc->GetTextExtent("My", &w, &h);
+	pdc->DrawText(s, 
+		      long(x_t / font_scaling_x),
+		      long(y_t / font_scaling_y) - h);
     } else {
 	pdc->GetTextExtent(s, &w, &h);
 	if ((y_t + h > 0 && y_t - h < clip.y_max - clip.y_min) ||
@@ -1202,6 +1278,7 @@ svxPrintout::WriteString(const char *s)
 	    fBlankPage = fFalse;
 	}
     }
+    pdc->SetUserScale(xsc, ysc);
 }
 
 void
@@ -1236,6 +1313,7 @@ svxPrintout::Pre()
 {
     font_labels = new wxFont(fontsize_labels,wxDEFAULT,wxNORMAL,wxNORMAL,false,fontname_labels,wxFONTENCODING_ISO8859_1);
     font_default = new wxFont(fontsize,wxDEFAULT,wxNORMAL,wxNORMAL,false,fontname,wxFONTENCODING_ISO8859_1);
+    current_font = font_default;
     pen_leg = new wxPen(colour_leg,0,wxSOLID);
     pen_surface_leg = new wxPen(colour_surface_leg,0,wxSOLID);
     pen_cross = new wxPen(colour_cross,0,wxSOLID);
