@@ -27,14 +27,15 @@ enum {
 	svx_SCALE,
 	svx_BEARING,
 	svx_TILT,
-	svx_ASPECT,
 	svx_LEGS,
 	svx_STATIONS,
 	svx_NAMES,
 	svx_BORDERS,
 	svx_BLANKS,
 	svx_INFOBOX,
-	svx_SCALEBAR
+	svx_SCALEBAR,
+	svx_PLAN,
+	svx_ELEV
 };
 
 class MainFrm;
@@ -42,8 +43,8 @@ struct layout;
 class wxComboBox;
 class wxStaticText;
 class wxSpinCtrl;
-class wxRadioBox;
 class wxCheckBox;
+class wxSpinEvent;
 
 class svxPrintDlg : public wxDialog {
 protected:
@@ -53,7 +54,6 @@ protected:
 	wxStaticText* m_tilttext;
 	wxSpinCtrl* m_bearing;
 	wxSpinCtrl* m_tilt;
-	wxRadioBox* m_aspect;
 	wxCheckBox* m_legs;
 	wxCheckBox* m_stations;
 	wxCheckBox* m_names;
@@ -75,8 +75,11 @@ protected:
 	~svxPrintDlg();
 	void OnPrint(wxCommandEvent& event); 
 	void OnPreview(wxCommandEvent& event);
+	void OnPlan(wxCommandEvent&);
+	void OnElevation(wxCommandEvent&);
+	void OnChangeSpin(wxSpinEvent& event); 
 	void OnChange(wxCommandEvent& event); 
-	void OnGetSettings(wxCommandEvent& event);
+	void SomethingChanged();
  private:
 	DECLARE_EVENT_TABLE()
 };
